package com.pengyifan.biocreativev;

import com.google.common.collect.BoundType;
import com.google.common.collect.Range;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraph;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraphVertex;
import java.util.List;
import java.util.stream.Collectors;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.base.Preconditions.checkNotNull;

public class DependencyGraphVertexFinder {

  private final DependencyGraph graph;
  private final Range<Integer> range;

  public DependencyGraphVertexFinder(DependencyGraph graph,
      Range<Integer> range) {
    checkNotNull(graph, "Dependency graph is null");
    checkNotNull(range, "Range is null");
    checkArgument(range.lowerBoundType() == BoundType.CLOSED,
        "The range lower bound should be closed.");
    checkArgument(range.upperBoundType() == BoundType.OPEN,
        "The range upper bound should be open.");
    this.graph = graph;
    this.range = range;
  }

  public List<DependencyGraphVertex> find() {
    return graph.vertexSet().stream()
        .filter(input -> {
          Range<Integer> inputRange = Range.closedOpen(
              input.getBeginPosition(),
              input.getEndPosition());
          return range.equals(inputRange);
        })
        .sorted((v1, v2) -> Integer.compare(
            v1.getBeginPosition(),
            v2.getBeginPosition()))
        .collect(Collectors.toList());
  }
}
