package com.pengyifan.biocreativev;

import com.pengyifan.brat.BratDocument;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraph;
import com.pengyifan.commons.io.BasenameUtils;
import com.pengyifan.kernel.MyProperties;
import com.pengyifan.kernel.allpathgraph.module.ApgConfiguration;
import com.pengyifan.kernel.allpathgraph.module.DefaultApgConfiguration;
import com.pengyifan.util.PropertiesLoader;
import edu.stanford.nlp.trees.Tree;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Collection;
import java.util.logging.LogManager;

public class PPIExtractorAnnotationBatch extends PPIExtractorBasicBatch {

  public static void main(String[] args)
      throws Exception {

    MyProperties prop = new MyProperties("biocreative5");
    String dataset = "dataset";
    Path root = prop.getTempDir().resolve(dataset);

    PPIExtractorAnnotationBatch s = new PPIExtractorAnnotationBatch(
        new DefaultApgConfiguration(),
        BasenameUtils.getTextBasenames(root.resolve("brat")));

    s.bratInputDir = root.resolve("brat");
    s.parsingInputDir = root.resolve("ptb2");
    s.outputDir = root.resolve("ppi-annotation");
    s.process();
  }

  protected PPIExtractorAnnotationBatch(ApgConfiguration config, Collection<String> basenames) {
    super(config, basenames);
  }

  @Override
  protected void preprocess()
      throws IOException {
    if (!Files.exists(outputDir)) {
      Files.createDirectory(outputDir);
    }
  }

  @Override
  protected void processFile(String basename)
      throws Exception {

    logger.info(String.format("Processing file: %s", basename));

    ppiDoc = new BratDocument();
    ppiDoc.setText(text);

    for (Tree tree : treebank) {
      if (tree.getLeaves().size() >= 100) {
        continue;
      }
      // get sdg
      DependencyGraph sdg = new SDGCreater(config.getGrammaticalStructureBuilder())
          .setDependencyType(config.getDependencyType())
          .setText(text)
          .setTree(tree)
          .create();
      // get ssdg
      DependencyGraph ssdg = new EDGCreater()
          .setText(text)
          .setA1Doc(a1Doc)
          .setCorefDoc(corefDoc)
          .setAbbrDoc(abbrDoc)
          .setDependencyGraph(sdg)
          .create();
      // extract
      PPIExtractorAnnotation ppi = new PPIExtractorAnnotation(ssdg, ppiDoc);
      ppi.extract();
    }
  }
}
