package com.pengyifan.biocreativev;

import com.pengyifan.brat.BratDocument;
import com.pengyifan.brat.BratEntity;
import com.pengyifan.brat.BratRelation;
import com.pengyifan.brat.io.BratIOUtils;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraph;
import com.pengyifan.commons.io.BasenameUtils;
import com.pengyifan.kernel.MyProperties;
import com.pengyifan.kernel.allpathgraph.module.ApgConfiguration;
import com.pengyifan.kernel.allpathgraph.module.DefaultApgConfiguration;
import edu.stanford.nlp.trees.Tree;
import org.apache.commons.io.FilenameUtils;

import java.io.IOException;
import java.nio.file.FileVisitResult;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.SimpleFileVisitor;
import java.nio.file.attribute.BasicFileAttributes;
import java.util.Collection;

public class PPIExtractorExtendedBatch extends PPIExtractorBasicBatch {

  public static void main(String[] args)
      throws Exception {

    MyProperties prop = new MyProperties("aimed");
    String dataset = "dataset";
    Path root = prop.getTempDir().resolve(dataset);

    PPIExtractorExtendedBatch s = new PPIExtractorExtendedBatch(
        new DefaultApgConfiguration(),
        BasenameUtils.getTextBasenames(root.resolve("brat")));

    s.bratInputDir = root.resolve("brat");
    s.parsingInputDir = root.resolve("ptb2");
    s.basicBratDir = root.resolve("ppi-basic");
    s.techBratDir = root.resolve("ppi-tech");
    s.corefDir = root.resolve("coref");
    s.abbrDir = root.resolve("abbr");
    s.outputDir = root.resolve("ppi-extended");
    s.triggerFile = prop.getDropboxResourcesDir().resolve("triggers/biocreative-all-triggers.txt");
    s.process();
  }

  private Path basicBratDir;
  private Path techBratDir;

  protected PPIExtractorExtendedBatch(ApgConfiguration config, Collection<String> basenames) {
    super(config, basenames);
  }

  @Override
  protected void preprocess() throws IOException {
    super.preprocess();
  }

  @Override
  protected void postprocessFile(String basename) throws Exception {
    BratDocument filteredDoc = new BratDocument();
    String prefix = basename; //basename.substring(0, basename.indexOf('_'));
    for(BratEntity entity: ppiDoc.getEntities()) {
      filteredDoc.addAnnotation(entity);
    }
    for(BratRelation rel: ppiDoc.getRelations()) {
      BratEntity e1 = ppiDoc.getEntity(rel.getArgId("Arg1"));
      BratEntity e2 = ppiDoc.getEntity(rel.getArgId("Arg2"));
      BratEntity trigger = ppiDoc.getEntity(rel.getArgId("Trigger"));
      if (find(prefix, e1, e2)) {
        filteredDoc.addAnnotation(rel);
//        if (!filteredDoc.containsId(e1.getId())) {
//          filteredDoc.addAnnotation(e1);
//        }
//        if (!filteredDoc.containsId(e2.getId())) {
//          filteredDoc.addAnnotation(e2);
//        }
//        if (!filteredDoc.containsId(trigger.getId())) {
//          filteredDoc.addAnnotation(trigger);
//        }
      }
    }
    ppiDoc = filteredDoc;
    super.postprocessFile(basename);
  }

  private boolean find(String prefix, BratEntity e1, BratEntity e2) throws IOException {
    // basic
    BratFileVisitor visitor = new BratFileVisitor(prefix, e1, e2);
    Files.walkFileTree(basicBratDir, visitor);
    if (visitor.isFound) {
      return true;
    }

    Files.walkFileTree(techBratDir, visitor);
    if (visitor.isFound) {
      return true;
    }

    return false;
  }

  @Override
  protected void processFile(String basename)
      throws Exception {

    logger.info(String.format("Processing file: %s", basename));

    ppiDoc = new BratDocument();
    ppiDoc.setText(text);

    for (Tree tree : treebank) {
      if (tree.getLeaves().size() >= 100) {
        continue;
      }
      // get sdg
      DependencyGraph sdg = new SDGCreater(config.getGrammaticalStructureBuilder())
          .setDependencyType(config.getDependencyType())
          .setText(text)
          .setTree(tree)
          .create();
      // get ssdg
      DependencyGraph ssdg = new EDGCreater()
          .setText(text)
          .setA1Doc(a1Doc)
          .setCorefDoc(corefDoc)
          .setAbbrDoc(abbrDoc)
          .setDependencyGraph(sdg)
          .create();
      // extract
      PPIExtractorExtended ppi = new PPIExtractorExtended(ssdg, ppiDoc, triggers);
      ppi.extract();
    }
  }

  private static class BratFileVisitor extends SimpleFileVisitor<Path> {
    String prefix;
    BratEntity target1;
    BratEntity target2;
    boolean isFound;

    public BratFileVisitor(String prefix, BratEntity target1, BratEntity target2) {
      this.prefix = prefix;
      this.target1 = target1;
      this.target2 = target2;
    }

    @Override
    public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) throws IOException {
      String basename = FilenameUtils.getBaseName(file.toString());
      if (basename.startsWith(prefix)) {
        BratDocument bratDocument = BratIOUtils.read(Files.newBufferedReader(file), basename);
        for(BratRelation rel: bratDocument.getRelations()) {
          if (!rel.containsRole("Arg2")) {
            continue;
          }
          BratEntity e1 = bratDocument.getEntity(rel.getArgId("Arg1"));
          BratEntity e2 = bratDocument.getEntity(rel.getArgId("Arg2"));
          if ((e1.getText().equalsIgnoreCase(target1.getText())
              && e2.getText().equalsIgnoreCase(target2.getText()))
              || (e2.getText().equalsIgnoreCase(target1.getText())
              && e1.getText().equalsIgnoreCase(target2.getText()))) {
            isFound = true;
            return FileVisitResult.TERMINATE;
          }
        }
      }
      return FileVisitResult.CONTINUE;
    }
  }
}
