package com.pengyifan.biocreativev;

import com.google.common.base.Splitter;
import com.google.common.collect.Lists;
import com.pengyifan.brat.BratDocument;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraph;
import com.pengyifan.commons.io.BasenameUtils;
import com.pengyifan.kernel.MyProperties;
import com.pengyifan.kernel.allpathgraph.module.ApgConfiguration;
import com.pengyifan.kernel.allpathgraph.module.DefaultApgConfiguration;
import com.pengyifan.util.PropertiesLoader;
import edu.stanford.nlp.trees.Tree;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.logging.LogManager;
import java.util.stream.Collectors;

public class PPIExtractorTechniqueBatch extends PPIExtractorBasicBatch {

  public static void main(String[] args)
      throws Exception {
    MyProperties prop = new MyProperties("aimed");
    String dataset = "dataset";
    Path root = prop.getTempDir().resolve(dataset);

    PPIExtractorTechniqueBatch s = new PPIExtractorTechniqueBatch(
        new DefaultApgConfiguration(),
        BasenameUtils.getTextBasenames(root.resolve("brat")));

    s.bratInputDir = root.resolve("brat");
    s.parsingInputDir = root.resolve("ptb2");
    s.corefDir = root.resolve("coref");
    s.abbrDir = root.resolve("abbr");
    s.outputDir = root.resolve("ppi-tech");
    s.techniqueFile = prop.getDropboxResourcesDir()
        .resolve("triggers/techinques.txt");
    s.process();
  }

  private static final Splitter splitter = Splitter.on(" ");
  Path techniqueFile;
  Set<List<String>> techniques;

  protected PPIExtractorTechniqueBatch(ApgConfiguration config, Collection<String> basenames) {
    super(config, basenames);
  }

  @Override
  protected void preprocess() throws IOException {
    if (!Files.exists(outputDir)) {
      Files.createDirectory(outputDir);
    }

    techniques = Files.readAllLines(techniqueFile).stream()
        .filter(s -> !s.isEmpty())
        .map(s -> Lists.newArrayList(splitter.split(s)))
        .collect(Collectors.toSet());
  }

  @Override
  protected void processFile(String basename)
      throws Exception {

    logger.info(String.format("Processing file: %s", basename));

    ppiDoc = new BratDocument();
    ppiDoc.setText(text);

    try {
      for (Tree tree : treebank) {
        if (tree.getLeaves().size() >= 100) {
          continue;
        }
        // get sdg
        DependencyGraph sdg = new SDGCreater(config.getGrammaticalStructureBuilder())
            .setDependencyType(config.getDependencyType())
            .setText(text)
            .setTree(tree)
            .create();
        // get ssdg
        DependencyGraph ssdg = new EDGCreater()
            .setText(text)
            .setA1Doc(a1Doc)
            .setCorefDoc(corefDoc)
            .setAbbrDoc(abbrDoc)
            .setDependencyGraph(sdg)
            .create();
        // extract
        PPIExtractorTechnique ppi = new PPIExtractorTechnique(ssdg, ppiDoc, techniques);
        ppi.extract();
      }
    } catch (Exception e) {

    }
  }
}
