package com.pengyifan.biocreativev;

import com.google.common.collect.Lists;
import com.pengyifan.bionlpst.utils.AnalysisHtmlPrinter;
import com.pengyifan.bionlpst.utils.ResultPrinter;
import com.pengyifan.brat.BratDocument;
import com.pengyifan.brat.BratEntity;
import com.pengyifan.brat.BratRelation;
import com.pengyifan.brat.io.BratIOUtils;
import com.pengyifan.brat.util.BratMerge;
import com.pengyifan.commons.io.AbstractBatchProcessor;
import com.pengyifan.commons.io.BasenameUtils;
import com.pengyifan.commons.math.PrecisionRecallStats;
import com.pengyifan.kernel.MyProperties;
import com.pengyifan.kernel.PairInstance;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.logging.Logger;
import java.util.stream.Collectors;

//import com.pengyifan.bionlpst.utils.ResultPrinter;
//import edu.stanford.nlp.stats.PrecisionRecallStats;

public class PPIInstanceEvaluate extends AbstractBatchProcessor {

  public static void main(String[] args)
      throws Exception {

    if (args.length == 0) {
      args = new String[1];
      args[0] = "aimed";
    }

    MyProperties prop = new MyProperties(args[0]);
    String dataset = "dataset";
    Path root = prop.getTempDir().resolve(dataset);

    PPIInstanceEvaluate s = new PPIInstanceEvaluate(
        BasenameUtils.getTextBasenames(root.resolve("brat")));
    s.bratInputDir = root.resolve("brat");
    s.ppiDir = root;
    s.shortestPathDir = root.resolve("shortestpath");
    s.outputDir = root.resolve("biocreative");

    s.process();
  }

  private Path bratInputDir;
  private Path ppiDir;
  private Path shortestPathDir;
  private Path outputDir;

  private String text;
  private BratDocument a1Doc;
  private BratDocument a2Doc;
  private BratDocument ppiDoc;

  private PrecisionRecallStats<BratRelation> totalStats;

  private List<PairInstance> totalTps;
  private List<PairInstance> totalFps;
  private List<PairInstance> totalFns;
  private List<PairInstance> totalTns;

  AnalysisHtmlPrinter tp;
  AnalysisHtmlPrinter fp;
  AnalysisHtmlPrinter fn;

  protected PPIInstanceEvaluate(Collection<String> basenames) {
    super(basenames);
  }

  @Override
  protected void preprocess() throws Exception {
    totalStats = new PrecisionRecallStats();
    totalTps = Lists.newArrayList();
    totalFps = Lists.newArrayList();
    totalFns = Lists.newArrayList();
    totalTns = Lists.newArrayList();

    if (!Files.exists(outputDir)) {
      Files.createDirectory(outputDir);
    }

    tp = new AnalysisHtmlPrinter(outputDir.resolve("tp5.html").toFile());
    fp = new AnalysisHtmlPrinter(outputDir.resolve("fp5.html").toFile());
    fn = new AnalysisHtmlPrinter(outputDir.resolve("fn5.html").toFile());
  }

  @Override
  protected void postprocess() throws Exception {
    ResultPrinter resultPrinter = new ResultPrinter();
    resultPrinter.printTitle();

    edu.stanford.nlp.stats.PrecisionRecallStats s = new
        edu.stanford.nlp.stats.PrecisionRecallStats(
        totalStats.getTP(),
        totalStats.getFP(),
        totalStats.getFN());
    resultPrinter.printRow("Total", s);

    //    PairInstancePrinter.writeInstances(outputDir.resolve("tp.pred"), totalTps);
    //    PairInstancePrinter.writeInstances(outputDir.resolve("fp.pred"), totalFps);
    //    PairInstancePrinter.writeInstances(outputDir.resolve("fn.pred"), totalFns);
    //    PairInstancePrinter.writeInstances(outputDir.resolve("tn.pred"), totalTns);

    // random
    //    Random rand = new Random(System.currentTimeMillis());
    //    List<PairInstance> randoms = Lists.newArrayList();
    //    for(int i=0; i<50; i++) {
    //      randoms.add(totalTps.get(rand.nextInt(totalTps.size())));
    //      randoms.add(totalTns.get(rand.nextInt(totalTns.size())));
    //      randoms.add(totalFps.get(rand.nextInt(totalFps.size())));
    //      randoms.add(totalFns.get(rand.nextInt(totalFns.size())));
    //    }
    //    Collections.shuffle(randoms);
    //    PairInstancePrinter.writeInstances(outputDir.resolve("randoms.pred"), randoms);
    tp.close();
    fp.close();
    fn.close();
  }

  //  private void write(Path file, List<BratRelation> relationList) {
  //    AnalysisHtmlPrinter p = new AnalysisHtmlPrinter(file.toFile());
  //    for(BratRelation relation: relationList) {
  //      String title = relation.getId()
  //      p.writeInstance();
  //    }
  //    p.close();
  //  }

  @Override
  protected void preprocessFile(String basename) throws Exception {
    // logger.info(String.format("Processing file: %s", basename));

    System.out.println("Processing " + basename);

    Path textFile = bratInputDir.resolve(basename + ".txt");
    text = new String(Files.readAllBytes(textFile));

    Path a1File = bratInputDir.resolve(basename + ".a1");
    a1Doc = BratIOUtils.read(Files.newBufferedReader(a1File), basename);

    Path a2File = bratInputDir.resolve(basename + ".a2");
    a2Doc = BratIOUtils.read(Files.newBufferedReader(a2File), basename);

    String[] ss = {"ppi-basic", "ppi-complex", "ppi-extended", "ppi-tech"};
    BratMerge m = new BratMerge();
    for (String s : ss) {
      Path p = ppiDir.resolve(s).resolve(basename + ".ann");
      BratDocument d = BratIOUtils.read(Files.newBufferedReader(p), basename);
      m.addDocument(d);
    }
    ppiDoc = m.getDoc();

    Path spFile = shortestPathDir.resolve(basename + ".sp");
    //    PathObjectReader r = new PathObjectReader(spFile);
    //    shortestPaths = r.readAll();
    //    r.close();

    a1Doc.setText(text);
    a2Doc.setText(text);
    //    for (PairInstance p : shortestPaths) {
    //      p.text = text;
    //    }
  }

  @Override
  protected void processFile(String basename) throws Exception {
    List<BratRelation> positives = a2Doc.getRelations().stream()
        .filter(r -> r.getType().equals("Interaction-True"))
        .collect(Collectors.toList());
    List<BratRelation> negatives = a2Doc.getRelations().stream()
        .filter(r -> r.getType().equals("Interaction-False"))
        .collect(Collectors.toList());

    boolean[] isInPred = new boolean[positives.size()];
    Arrays.fill(isInPred, false);

    for (BratRelation ppi : ppiDoc.getRelations()) {
      boolean isGold = false;
      for (int i = 0; i < positives.size(); i++) {
        BratRelation positive = positives.get(i);
        if (isEqual(ppi, positive)) {
          isGold = true;
          isInPred[i] = true;
          totalStats.incrementTP(positive);

          tp.writeInstance(basename, a1Doc.getText(), getRanges(positive, a1Doc));
          break;
        }
      }
      if (!isGold) {
        totalStats.incrementFP(ppi);
        fp.writeInstance(basename, a1Doc.getText(), getRanges(ppi, ppiDoc));
      }
    }

    for (int i = 0; i < isInPred.length; i++) {
      if (isInPred[i] == false) {
        totalStats.incrementFN(positives.get(i));
        fn.writeInstance(basename, a1Doc.getText(), getRanges(positives.get(i), a1Doc));
      }
    }

    // tn
    //    for(int i=0; i<negatives.size(); i++) {
    //      BratRelation negative = negatives.get(i);
    //      boolean isTn = true;
    //      for (BratRelation ppi : ppiDoc.getRelations()) {
    //        if (isEqual(ppi, negative)) {
    //          isTn = false;
    //          break;
    //        }
    //      }
    //      if (isTn) {
    ////        PairInstance tn = getShortestPath(negative);
    ////        tn.resultStatus = ResultStatus.TN;
    ////        totalTns.add(tn);
    //      }
    //    }
  }

  private List<AnalysisHtmlPrinter.EntityRange> getRanges(BratRelation relation,
      BratDocument document) {
    BratEntity predArg0 = document.getEntity(relation.getArgId("Arg1"));
    BratEntity predArg1 = document.getEntity(relation.getArgId("Arg2"));

    return Lists.newArrayList(
        new AnalysisHtmlPrinter.EntityRange(predArg0.totalSpan(), "Arg1", "A"),
        new AnalysisHtmlPrinter.EntityRange(predArg1.totalSpan(), "Arg2", "T")
    );
  }

  private boolean isEqual(BratRelation pred, BratRelation gold) {
    BratEntity predArg0 = ppiDoc.getEntity(pred.getArgId("Arg1"));
    BratEntity predArg1 = ppiDoc.getEntity(pred.getArgId("Arg2"));
    BratEntity goldArg0 = a1Doc.getEntity(gold.getArgId("Arg1"));
    BratEntity goldArg1 = a1Doc.getEntity(gold.getArgId("Arg2"));

    if (predArg0.totalSpan().equals(goldArg0.totalSpan())
        && predArg1.totalSpan().equals(goldArg1.totalSpan())) {
      return true;
    }
    if (predArg1.totalSpan().equals(goldArg0.totalSpan())
        && predArg0.totalSpan().equals(goldArg1.totalSpan())) {
      return true;
    }
    return false;
  }

  //  private PairInstance getShortestPath(BratRelation r, BratDocument doc) {
  //    BratEntity e1 = doc.getEntity(r.getArgId("arg0"));
  //    BratEntity e2 = doc.getEntity(r.getArgId("arg1"));
  //    for (PairInstance p : shortestPaths) {
  //      for(LinkedList<Object> list: p.listOfPathObjects) {
  //        DependencyGraphVertex v1 = (DependencyGraphVertex) list.getFirst();
  //        DependencyGraphVertex v2 = (DependencyGraphVertex) list.getLast();
  //        Range<Integer> r1 = Range.closedOpen(v1.getBeginPosition(), v1.getEndPosition());
  //        Range<Integer> r2 = Range.closedOpen(v2.getBeginPosition(), v2.getEndPosition());
  //        if ((r1.equals(e1.totalSpan()) && r2.equals(e2.totalSpan()))
  //            || (r2.equals(e1.totalSpan()) && r1.equals(e2.totalSpan()))) {
  //          return p;
  //        }
  //      }
  //    }
  //    checkArgument(false, "cannot find %s in %s", r.getId(), doc.getDocId());
  //    return null;
  //  }
  //
  //  private PairInstance getShortestPath(BratRelation r) {
  //    for (PairInstance p : shortestPaths) {
  //      if (p.id.equals(r.getId())) {
  //        return p;
  //      }
  //    }
  //    checkArgument(false, "cannot find %s", r.getId());
  //    return null;
  //  }
}
