package com.pengyifan.biocreativev;

import com.google.common.collect.Lists;
import com.pengyifan.bionlpst.utils.ResultPrinter;
import com.pengyifan.brat.BratDocument;
import com.pengyifan.brat.BratRelation;
import com.pengyifan.brat.io.BratIOUtils;
import com.pengyifan.commons.io.AbstractBatchProcessor;
import com.pengyifan.commons.io.BasenameUtils;
import com.pengyifan.kernel.MyProperties;
import com.pengyifan.kernel.PairInstance;
import com.pengyifan.kernel.PairInstancePrinter;
import edu.stanford.nlp.stats.PrecisionRecallStats;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;
import org.apache.commons.io.FileUtils;

public class PPIPassageEvaluate extends AbstractBatchProcessor {

  public static void main(String[] args)
      throws Exception {

    if (args.length == 0) {
      args = new String[1];
      args[0] = "aimed";
    }

    MyProperties prop = new MyProperties(args[0]);
    String dataset = "dataset";
    Path root = prop.getTempDir().resolve(dataset);

    PPIPassageEvaluate s = new PPIPassageEvaluate(
        BasenameUtils.getTextBasenames(root.resolve("brat")));
    s.bratInputDir = root.resolve("brat");
    s.ppiDir = root.resolve("ppi-basic");
    s.outputDir = root.resolve("biocreative");

    s.process();
  }

  //  protected static final Logger LOGGER = Logger.getAnonymousLogger();

  private Path bratInputDir;
  private Path ppiDir;
  private Path outputDir;

  private String text;
  private BratDocument a1Doc;
  private BratDocument a2Doc;
  private BratDocument ppiDoc;

  private PrecisionRecallStats totalStats;

  private List<PairInstance> totalTps;
  private List<String> totalFps;
  private List<PairInstance> totalFns;

  protected PPIPassageEvaluate(Collection<String> basenames) {
    super(basenames);
  }

  public void setOutputDir(Path outputDir) {
    this.outputDir = outputDir;
  }

  @Override
  protected void preprocess() throws Exception {
    totalStats = new PrecisionRecallStats();
    totalTps = Lists.newArrayList();
    totalFps = Lists.newArrayList();
    totalFns = Lists.newArrayList();
  }

  @Override
  protected void postprocess() throws Exception {
    ResultPrinter resultPrinter = new ResultPrinter();
    resultPrinter.printTitle();
    resultPrinter.printRow("Total", totalStats);

    PairInstancePrinter.writeInstances(outputDir.resolve("tp.pred"), totalTps);
    FileUtils.writeLines(outputDir.resolve("fp.pred").toFile(), totalFps);
    PairInstancePrinter.writeInstances(outputDir.resolve("fn.pred"), totalFns);
  }

  @Override
  protected void preprocessFile(String basename) throws Exception {
    // logger.info(String.format("Processing file: %s", basename));

    Path textFile = bratInputDir.resolve(basename + ".txt");
    text = new String(Files.readAllBytes(textFile));

    Path a1File = bratInputDir.resolve(basename + ".a1");
    a1Doc = BratIOUtils.read(Files.newBufferedReader(a1File), basename);

    Path a2File = bratInputDir.resolve(basename + ".a2");
    a2Doc = BratIOUtils.read(Files.newBufferedReader(a2File), basename);

    Path ppiFile = ppiDir.resolve(basename + ".ann");
    ppiDoc = BratIOUtils.read(Files.newBufferedReader(ppiFile), basename);

    a1Doc.setText(text);
    a2Doc.setText(text);
  }

  @Override
  protected void processFile(String basename) throws Exception {
    List<BratRelation> positives = a2Doc.getRelations().stream()
        .filter(r -> r.getType().equals("Interaction-True"))
        .collect(Collectors.toList());

    boolean goldHasPPI = !positives.isEmpty();
    boolean predHasPPI = !ppiDoc.getRelations().isEmpty();

    if (goldHasPPI && predHasPPI) {
      totalStats.incrementTP();
      //      totalTps.addAll(getSubShortestPaths(positives));
    } else if (!goldHasPPI && predHasPPI) {
      totalStats.incrementFP();
      totalFps.add(basename);
    } else if (goldHasPPI && !predHasPPI) {
      totalStats.incrementFN();
      //      totalFns.addAll(getSubShortestPaths(positives));
    }
  }

  //  private List<PairInstance> getSubShortestPaths(List<BratRelation> relations) {
  //    List<PairInstance> subShortestPaths = Lists.newArrayList();
  //    for (BratRelation r : relations) {
  //      boolean found = false;
  //      for (PairInstance p : shortestPaths) {
  //        if (p.id.equals(r.getId())) {
  //          subShortestPaths.add(p);
  //          found = true;
  //          break;
  //        }
  //      }
  //      checkArgument(found, "cannot find %s", r.getId());
  //    }
  //    return shortestPaths;
  //  }
}
