package com.pengyifan.biocreativev;

import com.pengyifan.commons.collections.dependencygraph.Dependency;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraph;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraphMatcher;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraphPattern;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraphVertex;
import com.pengyifan.commons.collections.dependencygraph.DependencyName;
import com.pengyifan.commons.collections.dependencygraph.NamedEntity;
import com.pengyifan.nlp.ling.string.EpitopeString;
import java.util.Collection;

public class ProteinSetter {

  private final DependencyGraph graph;
  private final Collection<DependencyGraphVertex> proteinVertexSet;

  public ProteinSetter(
      DependencyGraph graph,
      Collection<DependencyGraphVertex> proteinVertexSet) {
    super();
    this.graph = graph;
    this.proteinVertexSet = proteinVertexSet;
  }


  public void set() {
    setBrat();
    setIt();
    setIts();
    setEpitope();
    removeFamily();
  }

  private void removeFamily() {
    for (DependencyGraphVertex vertex : proteinVertexSet) {
      if (vertex.getLemma().endsWith("family")) {
        vertex.setNer(NamedEntity.UNKNOWN);
      }
    }
  }

  /**
   * Set Ner according to BratEntity's type
   */
  private void setBrat() {
    for (DependencyGraphVertex vertex : proteinVertexSet) {
      vertex.setNer(NamedEntity.PROTEIN_X);
      //      vertex.setTag(PartOfSpeech.NN);
    }
  }

  private void setEpitope() {
    for (DependencyGraphVertex vertex : graph.vertexSet()) {
      if (isEpitope(vertex.getLemma())) {
        vertex.setNer(NamedEntity.PROTEIN_X);
      }
    }
  }

  private boolean isEpitope(String s) {
    String ls = s.toLowerCase();
    for (String tag : EpitopeString.EPITOPE_TAG) {
      if (ls.endsWith("-" + tag) || ls.startsWith(tag + "-")) {
        return true;
      }
    }
    return false;
  }

  private void setIt() {
    for (DependencyGraphVertex vertex : graph.vertexSet()) {
      if (vertex.getWord().equalsIgnoreCase("it")
          || vertex.getWord().equalsIgnoreCase("its")) {
        vertex.setNer(NamedEntity.PROTEIN_IT);
      }
    }
  }

  // its < poss < protein
  private static final DependencyGraphPattern pattern = DependencyGraphPattern
      .of()
      // its
      .addVertex(0, v -> true)
      // protein
      .addVertex(1, v -> v.getWord().equalsIgnoreCase("protein"))
      .addEdge(1, 0, Dependency.ofPredicate(DependencyName.POSS));

  private void setIts() {
    DependencyGraphMatcher matcher = pattern.matcher(graph);
    while (matcher.find()) {
      DependencyGraphVertex s = matcher.get(1);
      s.setNer(NamedEntity.PROTEIN_X);
    }
  }

  /**
   * a/an/any/another vertex
   */
  private boolean hasIndefiniteDeterminer(DependencyGraphVertex vertex) {
    for (DependencyGraphVertex target : graph.outgoingEdgeTargetsOf(vertex)) {
      if ("a".equalsIgnoreCase(target.getWord())
          || "an".equalsIgnoreCase(target.getWord())
          || "any".equalsIgnoreCase(target.getWord())
          || "another".equalsIgnoreCase(target.getWord())) {
        return true;
      }
    }
    return false;
  }
}
