package com.pengyifan.biocreativev;

import com.pengyifan.commons.collections.dependencygraph.DependencyGraph;
import com.pengyifan.kernel.allpathgraph.singleprocess.DependencyGraphBuilder2;
import com.pengyifan.nlp.trees.DependencyBuilder;
import com.pengyifan.nlp.trees.DependencyType;
import com.pengyifan.nlp.trees.GrammaticalStructureBuilder;
import edu.stanford.nlp.semgraph.SemanticGraph;
import edu.stanford.nlp.trees.GrammaticalStructure;
import edu.stanford.nlp.trees.Tree;
import edu.stanford.nlp.trees.TypedDependency;

import java.util.Collection;
import java.util.logging.Logger;

import static com.google.common.base.Preconditions.checkNotNull;

public class SDGCreater {

  private static final Logger LOGGER = Logger.getAnonymousLogger();

  private final GrammaticalStructureBuilder gsBuilder;

  private String text;
  private DependencyType dependencyType;
  private Tree tree;
  private GrammaticalStructure grammaticalStructure;
  private DependencyGraph dependencyGraph;

  public SDGCreater(GrammaticalStructureBuilder gsBuilder) {
    this.gsBuilder = gsBuilder;
  }

  public SDGCreater setTree(Tree tree) {
    this.tree = tree;
    return this;
  }

  public SDGCreater setDependencyType(DependencyType dependencyType) {
    this.dependencyType = dependencyType;
    return this;
  }

  public DependencyGraph create() {
    setTreeWord();
    buildGrammaticalStructure();
    buildDependencyGraph();
    return dependencyGraph;
  }

  /**
   * Build dependency graph.
   */
  private void buildDependencyGraph() {
    LOGGER.finer("Start building dependency graph");
    checkNotNull(grammaticalStructure, "Grammatical structure is not set");
    checkNotNull(dependencyType, "Dependency type is not set");

    Collection<TypedDependency> dependencies = new DependencyBuilder(
        grammaticalStructure, dependencyType)
        .build();
    SemanticGraph semanticGraph = new SemanticGraph(dependencies);
    dependencyGraph = new DependencyGraphBuilder2(semanticGraph).build();
    dependencyGraph.checkHashCode();
  }

  private void buildGrammaticalStructure() {
    LOGGER.finer("Start building grammatical structure");
    checkNotNull(tree, "Tree is null");
    grammaticalStructure = gsBuilder.build(tree);
  }

  private void setTreeWord() {
    LOGGER.finer("Start setting treebank word");
    checkNotNull(tree, "Treebank is not set");
    checkNotNull(text, "Text is not set");
    try {
      new TreeWordSetter(tree, text).set();
    } catch (Exception e) {
      System.err.println(e.getMessage());
    }
  }

  public SDGCreater setText(String text) {
    this.text = text;
    return this;
  }
}
