package com.pengyifan.biocreativev.cmd;

import com.pengyifan.brat.BratDocument;
import com.pengyifan.brat.BratEntity;
import com.pengyifan.brat.io.BratIOUtils;
import com.pengyifan.kernel.MyProperties;
import java.io.IOException;
import java.nio.file.DirectoryIteratorException;
import java.nio.file.DirectoryStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;
import java.util.logging.Logger;
import java.util.stream.Collectors;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;

import static com.google.common.base.Preconditions.checkArgument;

public class Becas2BratAdapter {

  private static final Logger logger = Logger.getAnonymousLogger();

  public static void main(String[] args) throws IOException {
    MyProperties prop = new MyProperties("biocreative5");
    Path root = prop.getDropboxDir().resolve("dataset");
    try (DirectoryStream<Path> stream = Files.newDirectoryStream(
        root.resolve("origin/brat"), "*.txt")) {
      for (Path file : stream) {
        logger.info("Processing: " + file);

        String str = FileUtils.readFileToString(file.toFile());

        String basename = FilenameUtils.getBaseName(file.toString());
        Path annFile = root.resolve("becas").resolve(basename + ".ann");

        // check
        try {
          BratDocument bratDocument = BratIOUtils.read(Files.newBufferedReader(annFile), basename);
          List<BratEntity> entities = bratDocument.getEntities().stream()
              .sorted((a1, a2) -> Integer.compare(a1.beginPosition(), a2.beginPosition()))
              .collect(Collectors.toList());

          for (BratEntity ent : entities) {
            String token = str.substring(ent.beginPosition(), ent.endPosition());
            checkArgument(token.equals(ent.getText()), "Not equal [%s]:[%s]", token, ent);
          }

          // filter
          BratDocument fBratDocument = new BratDocument();
          bratDocument.getEntities().stream()
              .filter(e -> e.getType().equals("PRGE"))
              .forEach(e -> {
                e.setType("Protein");
                fBratDocument.addAnnotation(e);
              });

          Path fannFile = root.resolve("becas").resolve(basename + ".a1");
          BratIOUtils.write(Files.newBufferedWriter(fannFile), fBratDocument);
        } catch (IllegalArgumentException e) {
          e.printStackTrace();
        }
      }
    } catch (IOException | DirectoryIteratorException x) {
      System.err.println(x);
    }
  }
}
