package com.pengyifan.biocreativev.cmd;

import com.google.gson.Gson;
import com.pengyifan.kernel.MyProperties;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.net.URI;
import java.net.URISyntaxException;
import java.nio.file.DirectoryIteratorException;
import java.nio.file.DirectoryStream;
import java.nio.file.Files;
import java.nio.file.Path;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.io.IOUtils;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;

public class BecasNER {

  private static final String email = "ypeng@udel.edu";
  private static final String tool = "test";
  private static URI exportUri;

  static {
    try {
      exportUri = new URIBuilder()
          .setScheme("http")
          .setHost("bioinformatics.ua.pt")
          .setPath("/becas/api/text/export")
          .setParameter("email", email)
          .setParameter("tool", tool)
          .build();
    } catch (URISyntaxException e) {
      e.printStackTrace();
    }
  }

  private static final CloseableHttpClient httpClient = HttpClients.createDefault();

  public static void main(String[] args) throws IOException {
    MyProperties prop = new MyProperties("biocreative5");
    Path root = prop.getDropboxDir().resolve("dataset");
    try (DirectoryStream<Path> stream = Files.newDirectoryStream(
        root.resolve("origin/brat"), "*.txt")) {
      for (Path file : stream) {
        String basename = FilenameUtils.getBaseName(file.toString());
        Path annFile = root.resolve("becas").resolve(basename + ".ann");
        if (Files.exists(annFile)) {
          continue;
        }
        Thread r = new ThreadDemo(file, annFile);
        r.start();
        Thread.sleep(1000);
      }
    } catch (IOException | DirectoryIteratorException x) {
      System.err.println(x);
    } catch (InterruptedException e) {
      e.printStackTrace();
    }
  }

  private static class ThreadDemo extends Thread {

    private Path inputFile;
    private Path outputFile;

    ThreadDemo(Path inputFile, Path outputFile) {
      this.inputFile = inputFile;
      this.outputFile = outputFile;
    }

    @Override
    public void run() {
      System.out.printf("process %s\n", inputFile.toString());
      try {
        String str = FileUtils.readFileToString(inputFile.toFile());

        System.out.println("sending " + exportUri);
        HttpPost request = new HttpPost(exportUri);
        request.addHeader("content-type", "application/json");

        Gson gson = new Gson();
        Becas becas = new Becas("a1", str);
        String params = gson.toJson(becas);
        System.out.println(params);

        request.setEntity(new StringEntity(params));
        CloseableHttpResponse response = httpClient.execute(request);

        // handle response here
        String data = getData(response);
        response.close();

        FileUtils.write(outputFile.toFile(), data);
      } catch (Exception ex) {
        ex.printStackTrace();
      }
      System.out.printf("Done %s\n", inputFile.toString());
    }

    private String getData(CloseableHttpResponse response) throws IOException {
      InputStream is = response.getEntity().getContent();
      StringWriter writer = new StringWriter();
      IOUtils.copy(is, writer, "utf8");
      return writer.toString();
    }

    private static class Becas {
      private String format;
      private String text;

      public Becas(String format, String text) {
        this.format = format;
        this.text = text;
      }
    }
  }
}
