package com.pengyifan.biocreativev.cmd;

import com.google.common.collect.Sets;
import com.pengyifan.bioc.BioCAnnotation;
import com.pengyifan.bioc.BioCCollection;
import com.pengyifan.bioc.BioCNode;
import com.pengyifan.bioc.BioCPassage;
import com.pengyifan.bioc.BioCRelation;
import com.pengyifan.bioc.io.BioCCollectionReader;
import com.pengyifan.bioc.util.BioCPassageIterator;
import java.io.BufferedWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Set;
import javax.xml.stream.XMLStreamException;
import org.apache.commons.io.FilenameUtils;
import org.javatuples.Triplet;

import static com.google.common.base.Preconditions.checkNotNull;

public class BioC2BioGrid {
  public static void main(String[] args) throws IOException, XMLStreamException {
    Path biocDir = Paths.get("tmp/biocreative5/dataset/v5/");

    Set<Triplet<String, String, String>> set = Sets.newTreeSet();

    for(Path biocFile: Files.newDirectoryStream(biocDir)) {
      System.out.println(biocFile);

      BioCCollectionReader reader = new BioCCollectionReader(biocFile.toFile());
      BioCCollection collection = reader.readCollection();
      reader.close();

      String docId = FilenameUtils.getBaseName(biocFile.toString());

      BioCPassageIterator itr = new BioCPassageIterator(collection);
      while (itr.hasNext()) {
        BioCPassage passage = itr.next();
        for(BioCRelation relation: passage.getRelations()) {
          BioCAnnotation arg1 = null;
          BioCAnnotation arg2 = null;
          for(BioCNode node: relation.getNodes()) {
            if (node.getRole().equals("Arg1")) {
              arg1 = passage.getAnnotation(node.getRefid()).get();
            }
            if (node.getRole().equals("Arg2")) {
              arg2 = passage.getAnnotation(node.getRefid()).get();
            }
          }
          checkNotNull(arg1, "Arg1 is null");
          checkNotNull(arg2, "Arg2 is null");
          set.add(Triplet.with(docId, arg1.getText().get(), arg2.getText().get()));
        }
      }
    }

    Path file = Paths.get("tmp/biocreative5/dataset/pred2.csv");
    BufferedWriter writer = Files.newBufferedWriter(file);
    for(Triplet<String, String, String> t: set) {
      writer.append(String.format("%s\t%s\t%s\n", t.getValue0(), t.getValue1(), t.getValue2()));
    }
    writer.close();
  }
}
