package com.pengyifan.biocreativev.cmd;

import com.google.common.base.Joiner;
import com.google.common.collect.Maps;
import com.google.common.collect.Range;
import com.pengyifan.bioc.BioCAnnotation;
import com.pengyifan.bioc.BioCCollection;
import com.pengyifan.bioc.BioCNode;
import com.pengyifan.bioc.BioCPassage;
import com.pengyifan.bioc.BioCRelation;
import com.pengyifan.bioc.util.BioCPassageIterator;
import com.pengyifan.bioc.util.BioCUtils;
import com.pengyifan.biocreativev.PassageType;
import com.pengyifan.nlp.ling.string.EpitopeString;
import java.io.BufferedWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.EnumSet;
import java.util.Map;
import java.util.Optional;
import javax.xml.stream.XMLStreamException;
import org.apache.commons.io.FilenameUtils;
import org.javatuples.Triplet;

import static com.google.common.base.Preconditions.checkNotNull;

public class BioC2BioGridTitle {

  public static void main(String[] args) throws IOException, XMLStreamException {
    Path idir = Paths.get("tmp/biocreative5/dataset/v5/");


    Map<Triplet<String, String, String>, EnumSet<PassageType>> map = Maps.newTreeMap();

    for (Path biocFile : Files.newDirectoryStream(idir)) {
      System.out.println(biocFile);

      String docId = FilenameUtils.getBaseName(biocFile.toString());
      if (!Documents20.contains(docId)) {
        continue;
      }

      BioCCollection collection = BioCUtils.readCollection(biocFile);
      Range<Integer> resultRange = getResultRange(collection);

      BioCPassageIterator itr = new BioCPassageIterator(collection);
      while (itr.hasNext()) {
        BioCPassage passage = itr.next();
        PassageType type = PassageType.getPassageType(passage);
        boolean isTitle = type.isTitle();

        if (isTitle && resultRange != null && !resultRange.contains(passage.getOffset())) {
          type = PassageType.UNKNOWN;
        }

        for (BioCRelation relation : passage.getRelations()) {
          BioCAnnotation arg1 = null;
          BioCAnnotation arg2 = null;
          boolean isNeg = false;
          for (BioCNode node : relation.getNodes()) {
            if (node.getRole().equals("Arg1")) {
              arg1 = passage.getAnnotation(node.getRefid()).get();
            }
            if (node.getRole().equals("Arg2")) {
              arg2 = passage.getAnnotation(node.getRefid()).get();
            }
            if (node.getRole().equals("Neg")) {
              isNeg = true;
            }
          }
          if (isNeg) {
            continue;
          }

          checkNotNull(arg1, "Arg1 is null");
          checkNotNull(arg2, "Arg2 is null");

          if (EpitopeString.isEpitopeTag(arg1.getText().get().toLowerCase())
              || EpitopeString.isEpitopeTag(arg2.getText().get().toLowerCase())) {
            continue;
          }

          Triplet<String, String, String> t = Triplet.with(docId,
              arg1.getText().get().toLowerCase(),
              arg2.getText().get().toLowerCase());
          if (t.getValue1().compareTo(t.getValue2()) > 0) {
            t = Triplet.with(docId, t.getValue2(), t.getValue1());
          }

          if (map.containsKey(t)) {
            map.get(t).add(type);
          } else {
            map.put(t, EnumSet.of(type));
          }
        }
      }
    }

    Path file = Paths.get("tmp/biocreative5/dataset/pred-20-title.csv");
    BufferedWriter writer = Files.newBufferedWriter(file);
    for (Triplet<String, String, String> t : map.keySet()) {
      writer.append(String.format("%s\t%s\t%s\t%s\n", t.getValue0(), t.getValue1(), t.getValue2(),
          Joiner.on('|').join(map.get(t))));
    }
    writer.close();
  }

  private static Range<Integer> getResultRange(BioCCollection collection) {
    int start = -1;
    int end = -1;
    BioCPassageIterator itr = new BioCPassageIterator(collection);
    while (itr.hasNext()) {
      BioCPassage passage = itr.next();
      Optional<String> maybeType = passage.getInfon("type");
      if (maybeType.isPresent()) {
        String type = maybeType.get();
        if (type.equals("title_1")) {
          String text = passage.getText().orElse("");
          if (text.toLowerCase().startsWith("result")) {
            start = passage.getOffset();
          } else if (start != -1) {
            end = passage.getOffset();
          }
        }
      }
    }
    if (start != -1 && end != -1) {
      return Range.closedOpen(start, end);
    } else {
      return null;
    }
  }
}
