package com.pengyifan.biocreativev.cmd;

import com.google.common.collect.Iterables;
import com.google.common.collect.Maps;
import com.pengyifan.bioc.BioCAnnotation;
import com.pengyifan.bioc.BioCCollection;
import com.pengyifan.bioc.BioCLocation;
import com.pengyifan.bioc.BioCSentence;
import com.pengyifan.bioc.io.BioCCollectionReader;
import com.pengyifan.bioc.util.BioCSentenceIterator;
import com.pengyifan.brat.BratDocument;
import com.pengyifan.brat.BratEntity;
import com.pengyifan.brat.io.BratIOUtils;
import com.pengyifan.commons.convert.BiocToBratConverter;
import com.pengyifan.commons.io.AbstractBatchProcessor;
import com.pengyifan.kernel.MyProperties;
import java.io.IOException;
import java.nio.file.DirectoryStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Collection;
import java.util.Map;
import java.util.Map.Entry;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.xml.stream.XMLStreamException;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;

import static com.google.common.base.Preconditions.checkArgument;

public class BioC2BratNer286 extends AbstractBatchProcessor {

  public static void main(String[] args)
      throws Exception {
    MyProperties prop = new MyProperties("biocreative5");
    Path root = prop.getDropboxDir().resolve("sample");

    BioC2BratNer286 b = new BioC2BratNer286(BiocreativeUtils.getTextBasenames("sample"));
    b.textDir = root.resolve("origin/brat");
    b.biocDir = root.resolve("team312/bioc");
    b.bioc2Dir = root.resolve("team312/bioc2");
    b.annDir = root.resolve("team312/brat");
    b.process();
  }

  private Path textDir;
  private Path biocDir;
  private Path bioc2Dir;
  private Path annDir;

  public BioC2BratNer286(Collection<String> basenames) {
    super(basenames);
  }

  @Override
  protected void processFile(String basename) throws Exception {
    System.err.printf("process %s\n", basename);

    Path biocFile = biocDir.resolve(basename + ".xml");

    // fixed
    String xmlStr = FileUtils.readFileToString(biocFile.toFile());
    xmlStr = fix(xmlStr);

    Path bioc2File = bioc2Dir.resolve(basename + ".xml");
    FileUtils.write(bioc2File.toFile(), xmlStr);

    convert(basename, bioc2File);
  }

  private static final Pattern p = Pattern.compile("<annotation id=\"(.*?)\">");

  private String fix(String s) {
    StringBuffer buffer = new StringBuffer();
    int i = 0;
    Matcher m = p.matcher(s);
    while (m.find()) {
      m.appendReplacement(buffer, "<annotation id=\"O" + i++ + "\">");
    }
    m.appendTail(buffer);
    return buffer.toString();
  }

  private void convert(String basename, Path inputFile) throws
      IOException,
      XMLStreamException {

    BiocToBratConverter converter = new BiocToBratConverter();

    try {
      BioCCollectionReader biocReader = new BioCCollectionReader(inputFile.toFile());
      BioCCollection collection = biocReader.readCollection();
      biocReader.close();

      Map<Integer, String> passages = Maps.newHashMap();
      Map<Integer, BratDocument> bratDocuments = Maps.newHashMap();
      DirectoryStream<Path> stream = Files.newDirectoryStream(textDir, basename + "_*.txt");
      for (Path file : stream) {
        String b = FilenameUtils.getBaseName(file.toString());
        int offset = Integer.parseInt(b.substring(b.lastIndexOf('_') + 1));
        passages.put(offset, FileUtils.readFileToString(file.toFile()));
        bratDocuments.put(offset, new BratDocument());
      }

      int index = 0;
      BioCSentenceIterator sentenceIterator = new BioCSentenceIterator(collection);
      while (sentenceIterator.hasNext()) {
        BioCSentence bioCSentence = sentenceIterator.next();
        Entry<Integer, String> entry = findPassage(bioCSentence.getOffset(), passages);
        int passageOffset = entry.getKey();
        String text = entry.getValue();

        for (BioCAnnotation bioCAnnotation : bioCSentence.getAnnotations()) {
          bioCAnnotation.setID("T" + index++);
          // check
          String expected = bioCAnnotation.getText().get();
          BioCLocation loc = Iterables.getOnlyElement(bioCAnnotation.getLocations());

          int beginPosition = loc.getOffset() - passageOffset;
          int endPosition = loc.getOffset() - passageOffset + loc.getLength();

          String actual = text.substring(beginPosition, endPosition);
          try {
            checkArgument(expected.equals(actual), "expect [%s] is not equal to actual [%s]\n%s",
                expected, actual, bioCAnnotation);
          } catch (Exception e) {
            e.printStackTrace();
            continue;
          }
          BratEntity bratEntity = converter.convert(bioCAnnotation);
          BratEntity shiftedBratEntity = BratEntity.shift(bratEntity, -passageOffset);
          bratDocuments.get(passageOffset).addAnnotation(shiftedBratEntity);
        }
      }

      for(Entry<Integer, BratDocument> entry: bratDocuments.entrySet()) {
        Path annFile = BiocreativeUtils.getFile(annDir, basename, entry.getKey(), "ann");
        BratIOUtils.write(Files.newBufferedWriter(annFile), entry.getValue());
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  private Entry<Integer, String> findPassage(int offset, Map<Integer, String> passages) {
    for (Entry<Integer, String> entry : passages.entrySet()) {
      if (entry.getKey() <= offset && offset < entry.getKey() + entry.getValue().length()) {
        return entry;
      }
    }
    checkArgument(false, "Cannot find sentences: %s", offset);
    return null;
  }
}
