package com.pengyifan.biocreativev.cmd;

import com.pengyifan.bioc.BioCCollection;
import com.pengyifan.bioc.BioCPassage;
import com.pengyifan.bioc.io.BioCCollectionReader;
import com.pengyifan.bioc.util.BioCPassageIterator;
import com.pengyifan.commons.io.AbstractBatchProcessor;
import com.pengyifan.kernel.MyProperties;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Collection;
import org.apache.commons.io.FileUtils;

public class BioC2Text extends AbstractBatchProcessor {

  public static void main(String[] args)
      throws Exception {

    MyProperties prop = new MyProperties("biocreative5");
    String dataset = "dataset";

    BioC2Text b = new BioC2Text(BiocreativeUtils.getBioCBasenames(dataset));
    b.biocDir = prop.getDropboxDir().resolve(dataset).resolve("origin/bioc");
    b.textDir = prop.getDropboxDir().resolve(dataset).resolve("origin/brat");
    b.process();
  }

  private Path biocDir;
  private Path textDir;

  private BioCCollection collection;

  public BioC2Text(Collection<String> basenames) {
    super(basenames);
  }

  @Override
  public void preprocess() throws Exception {
    if (!Files.exists(textDir)) {
      Files.createDirectory(textDir);
    }
  }

  @Override
  public void preprocessFile(String basename) throws Exception {
    Path biocFile = biocDir.resolve(basename + ".xml");
    BioCCollectionReader reader = new BioCCollectionReader(biocFile.toFile());
    collection = reader.readCollection();
    reader.close();
  }

  @Override
  public void processFile(String basename) throws Exception {
    BioCPassageIterator passageIterator = new BioCPassageIterator(collection);
    while (passageIterator.hasNext()) {
      BioCPassage passage = passageIterator.next();
      Path textFile = BiocreativeUtils.getFile(textDir, basename, passage.getOffset(), "txt");
      if (passage.getText().isPresent()) {
        FileUtils.write(textFile.toFile(), passage.getText().get());
      } else {
        FileUtils.write(textFile.toFile(), "");
      }
    }
  }
}
