package com.pengyifan.biocreativev.cmd;

import com.google.common.collect.Maps;
import com.pengyifan.brat.BratAnnotation;
import com.pengyifan.brat.BratDocument;
import com.pengyifan.brat.BratEntity;
import com.pengyifan.brat.BratRelation;
import com.pengyifan.brat.io.BratIOUtils;
import com.pengyifan.brat.util.BratMerge;
import com.pengyifan.commons.io.AbstractBatchProcessor;
import com.pengyifan.kernel.MyProperties;
import org.apache.commons.io.FileUtils;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

public class BratDemonstration extends AbstractBatchProcessor {


  public static void main(String[] args)
      throws Exception {
    MyProperties prop = new MyProperties("biocreative5");
    BratDemonstration b = new BratDemonstration(BiocreativeUtils.getAllTextBasenames("sample"));
    b.a1Dir = prop.getTempDir().resolve("brat");
    b.ppiDir = prop.getTempDir().resolve("ppi");
    b.outputDir = prop.getTempDir().resolve("brat");

    b.process();
  }

  private Path a1Dir;
  private Path ppiDir;
  private Path outputDir;

  public BratDemonstration(Collection<String> basenames) {
    super(basenames);
  }

  @Override
  protected void processFile(String basename) throws Exception {
    System.out.printf("process %s\n", basename);

    BratMerge merge = new BratMerge();

    Path a1File = a1Dir.resolve(basename + ".a1");
    merge(basename, a1File, merge);

    Path ppiFile = ppiDir.resolve(basename + ".ann");
    merge(basename, ppiFile, merge);

    BratDocument doc = merge.getDoc();
    Path outputFile = outputDir.resolve(basename + ".ann");

    doc = reorder(doc);

    StringBuilder sb = new StringBuilder();

    List<BratEntity> entityList = doc.getEntities().stream()
        .sorted((e1, e2) -> {
          Integer i1 = Integer.parseInt(e1.getId().substring(1));
          Integer i2 = Integer.parseInt(e2.getId().substring(1));
          return Integer.compare(i1, i2);
        })
        .collect(Collectors.toList());
    for(BratEntity ent: entityList) {
      sb.append(ent).append('\n');
    }

    List<BratRelation> relationList = doc.getRelations().stream()
        .sorted((e1, e2) -> {
          Integer i1 = Integer.parseInt(e1.getId().substring(1));
          Integer i2 = Integer.parseInt(e2.getId().substring(1));
          return Integer.compare(i1, i2);
        })
        .collect(Collectors.toList());
    for(BratRelation rel: relationList) {
      String s = String.format("%s\t%s Arg0:%s Arg1:%s", rel.getId(), rel.getType(),
          rel.getArgId("arg0"), rel.getArgId("arg1"));
      sb.append(s).append('\n');
    }
    FileUtils.write(outputFile.toFile(), sb);
  }

  private void merge(String basename, Path file, BratMerge merge) throws IOException {
    if (Files.exists(file)) {
      BratDocument doc = BratIOUtils.read(Files.newBufferedReader(file), basename);
      merge.addDocument(doc);
    } else {
      System.err.printf("%s does not exsist.\n", file);
    }
  }

  private BratDocument reorder(BratDocument doc) {
    BratDocument newDoc = new BratDocument();
    newDoc.setDocId(doc.getDocId());
    newDoc.setText(doc.getText());

    List<BratEntity> entityList = doc.getEntities().stream()
        .sorted((e1, e2) -> Integer.compare(e1.beginPosition(), e2.beginPosition()))
        .collect(Collectors.toList());

    // old, new
    Map<String, String> map = Maps.newHashMap();
    int i = 0;
    for(BratEntity entity: entityList) {
      BratEntity newEntity = new BratEntity(entity);
      newEntity.setId("T" + i++);
      map.put(entity.getId(), newEntity.getId());
      newDoc.addAnnotation(newEntity);
    }
    for(BratRelation relation: doc.getRelations()) {
      BratRelation newRelation = new BratRelation(relation);
      for(String key: newRelation.getArguments().keySet()) {
        newRelation.putArgument(key, map.get(relation.getArgId(key)));
      }
      newDoc.addAnnotation(newRelation);
    }
    return newDoc;
  }
}
