package com.pengyifan.biocreativev.cmd;

import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import com.pengyifan.brat.BratDocument;
import com.pengyifan.brat.BratEntity;
import com.pengyifan.brat.BratRelation;
import com.pengyifan.brat.io.BratIOUtils;
import com.pengyifan.brat.util.BratMerge;
import com.pengyifan.commons.io.AbstractBatchProcessor;
import com.pengyifan.commons.nio.BatchProcessor;
import com.pengyifan.commons.nio.SimpleBasenameVisitor;
import com.pengyifan.kernel.MyProperties;
import com.pengyifan.nlp.ling.string.EpitopeString;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang.StringEscapeUtils;

import java.io.IOException;
import java.nio.file.FileVisitResult;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.SimpleFileVisitor;
import java.nio.file.attribute.BasicFileAttributes;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

public class BratNerExtend extends SimpleBasenameVisitor {

  public static void main(String[] args)
      throws Exception {
    MyProperties prop = new MyProperties("biocreative5");
    String dataset = "dataset";

    Path root = prop.getDropboxDir().resolve(dataset);

    BratNerExtend b = new BratNerExtend();
    b.textDir = root.resolve("origin/brat");
    b.nerDir = root.resolve("ner");
    b.outputDir = root.resolve("ner");

    BatchProcessor.walk(
        BiocreativeUtils.getAllTextBasenames(dataset),
        b
    );
  }

  private static final Logger logger = Logger.getAnonymousLogger();
  private Path textDir;
  private Path nerDir;
  private Path outputDir;

  BratDocument nerDoc;
  String text;

  @Override
  public void preprocessFile(String basename) throws Exception {
    Path textFile = textDir.resolve(basename + ".txt");
    text = FileUtils.readFileToString(textFile.toFile());

    Path nerFile = nerDir.resolve(basename + ".a1");
    nerDoc = BratIOUtils.read(Files.newBufferedReader(nerFile), basename);
  }

  @Override
  public void processFile(String basename) throws Exception {
    String prefix = basename.substring(0, basename.indexOf('_'));
    BratFileVisitor visitor = new BratFileVisitor(prefix);
    Files.walkFileTree(nerDir, visitor);
    if (visitor.entities.isEmpty()) {
      return;
    }
    int id = nerDoc.getEntities().size();
    for(BratEntity entity: visitor.entities) {
      id = addEntities(entity.getText(), id);
      for(String tag: EpitopeString.EPITOPE_TAG) {
        id = addEntities(entity.getText() + "-" + tag.toUpperCase(), id);
        id = addEntities(tag.toUpperCase() + "-" + entity.getText(), id);
      }
    }
  }

  @Override
  public void postprocessFile(String basename) throws Exception {
    Path outputFile = outputDir.resolve(basename + ".a1");
    BratIOUtils.write(Files.newBufferedWriter(outputFile), nerDoc);
  }

  private int addEntities(String word, int id) {
    try {
      Pattern p = Pattern.compile("\\b(" + StringEscapeUtils.escapeJava(word) + ")\\b");
      Matcher m = p.matcher(text);
      while (m.find()) {
        BratEntity ann = new BratEntity();
        ann.setId("T" + id++);
        ann.setType("Protein");
        ann.setText(m.group(1));
        ann.addSpan(m.start(1), m.end(1));
        nerDoc.addAnnotation(ann);
      }
    } catch (PatternSyntaxException e) {
      logger.warning(e.getMessage());
    }
    return id;
  }

  private static class BratFileVisitor extends SimpleFileVisitor<Path> {
    String prefix;
    List<BratEntity> entities;

    public BratFileVisitor(String prefix) {
      this.prefix = prefix;
      entities = Lists.newArrayList();
    }

    @Override
    public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) throws IOException {
      String basename = FilenameUtils.getBaseName(file.toString());
      if (basename.startsWith(prefix)) {
        BratDocument bratDocument = BratIOUtils.read(Files.newBufferedReader(file), basename);
        for(BratEntity entity: bratDocument.getEntities()) {
          entities.add(entity);
        }
      }
      return FileVisitResult.CONTINUE;
    }
  }
}
