package com.pengyifan.biocreativev.cmd;

import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import com.pengyifan.brat.BratDocument;
import com.pengyifan.brat.io.BratIOUtils;
import com.pengyifan.brat.util.BratMerge;
import com.pengyifan.commons.io.AbstractBatchProcessor;
import com.pengyifan.kernel.MyProperties;

import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.logging.Logger;

public class BratNerMerge extends AbstractBatchProcessor {

  public static void main(String[] args)
      throws Exception {
    MyProperties prop = new MyProperties("biocreative5");
    String dataset = "dataset";

    Path root = prop.getDropboxDir().resolve(dataset);

    BratNerMerge b = new BratNerMerge(BiocreativeUtils.getAllTextBasenames(dataset));
    b.bratDirs = Lists.newArrayList();
    b.bratDirs.add(root.resolve("team0/brat"));
    b.bratDirs.add(root.resolve("team286/brat"));
    b.bratDirs.add(root.resolve("team309/brat"));
    b.bratDirs.add(root.resolve("team312/brat"));
    b.bratDirs.add(root.resolve("becas"));
    b.outputDir = root.resolve("ner");

    b.process();
  }

  private static final Logger logger = Logger.getAnonymousLogger();
  private List<Path> bratDirs;
  private Path outputDir;

  public BratNerMerge(Collection<String> basenames) {
    super(basenames);
  }

  @Override
  protected void preprocess() throws Exception {
    if (!Files.exists(outputDir)) {
      Files.createDirectory(outputDir);
    }
  }

  @Override
  protected void processFile(String basename) throws Exception {
    logger.info("process " + basename);

    BratMerge merge = new BratMerge();
    for (Path bratDir : bratDirs) {
      Path bratFile = bratDir.resolve(basename + ".a1");
      if (Files.exists(bratFile)) {
        BratDocument doc = BratIOUtils.read(Files.newBufferedReader(bratFile), basename);
        merge.addDocument(doc);
      } else {
        logger.warning(String.format("%s does not exsist.\n", bratFile));
      }
    }
    BratDocument doc = merge.getDoc();
    Path outputFile = outputDir.resolve(basename + ".a1");
    BratIOUtils.write(Files.newBufferedWriter(outputFile), doc);
  }
}
