package com.pengyifan.biocreativev.cmd;

import com.google.common.collect.Lists;
import com.pengyifan.brat.BratDocument;
import com.pengyifan.brat.BratEntity;
import com.pengyifan.brat.BratRelation;
import com.pengyifan.brat.io.BratIOUtils;
import com.pengyifan.brat.util.BratMerge;
import com.pengyifan.commons.io.AbstractBatchProcessor;
import com.pengyifan.kernel.MyProperties;
import org.apache.commons.io.FilenameUtils;

import java.io.IOException;
import java.nio.file.FileVisitResult;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.SimpleFileVisitor;
import java.nio.file.attribute.BasicFileAttributes;
import java.util.Collection;
import java.util.List;
import java.util.Optional;
import java.util.logging.Logger;

public class BratPpiMerge extends AbstractBatchProcessor {


  public static void main(String[] args)
      throws Exception {
    MyProperties prop = new MyProperties("biocreative5");
    String dataset = "dataset";
    Path root = prop.getTempDir().resolve(dataset);

    BratPpiMerge b = new BratPpiMerge(BiocreativeUtils.getAllTextBasenames(dataset));
//    b.jnlpDir = root.resolve("brat");
    b.basicBratDir = root.resolve("ppi-basic");
    b.extendedBratDir = root.resolve("ppi-extended");
    b.techBratDir = root.resolve("ppi-tech");
    b.annotationBratDir = root.resolve("ppi-annotation");
    b.propagateBratDir = root.resolve("ppi-propagate");
    b.negBratDir = root.resolve("ppi-neg");
    b.outputDir = root.resolve("ppi");

    b.process();
  }

  private final static Logger logger = Logger.getAnonymousLogger();
//  private Path jnlpDir;
  private Path basicBratDir;
  private Path extendedBratDir;
  private Path techBratDir;
  private Path annotationBratDir;
  private Path propagateBratDir;
  private Path negBratDir;

  private Path outputDir;

  private BratDocument basicDoc;
  private BratDocument extendedDoc;
  private BratDocument techDoc;
  private BratDocument annotationDoc;
  private BratDocument propagateDoc;
  private BratDocument negDoc;

  public BratPpiMerge(Collection<String> basenames) {
    super(basenames);
  }

  @Override
  protected void preprocess() throws Exception {
    super.preprocess();
    if (!Files.exists(outputDir)) {
      Files.createDirectory(outputDir);
    }
  }

  @Override
  protected void preprocessFile(String basename) throws Exception {
    super.preprocessFile(basename);

//    Path textFile = jnlpDir.resolve(basename + ".txt");
//    lines = Files.readAllLines(textFile);

    Path basicFile = basicBratDir.resolve(basename + ".ann");
    if (Files.exists(basicFile)) {
      basicDoc = BratIOUtils.read(Files.newBufferedReader(basicFile), basename);
    } else {
      logger.warning(String.format("%s does not exist.\n", basicFile));
      basicDoc = new BratDocument();
    }

    Path extendedFile = extendedBratDir.resolve(basename + ".ann");
    if (Files.exists(extendedFile)) {
      extendedDoc = BratIOUtils.read(Files.newBufferedReader(extendedFile), basename);
    } else {
      logger.warning(String.format("%s does not exist.\n", extendedFile));
      extendedDoc = new BratDocument();
    }

    Path techFile = techBratDir.resolve(basename + ".ann");
    if (Files.exists(techFile)) {
      techDoc = BratIOUtils.read(Files.newBufferedReader(techFile), basename);
    } else {
      logger.warning(String.format("%s does not exist.\n", techFile));
      techDoc = new BratDocument();
    }

    Path annotationFile = annotationBratDir.resolve(basename + ".ann");
    if (Files.exists(annotationFile)) {
      annotationDoc = BratIOUtils.read(Files.newBufferedReader(annotationFile), basename);
    } else {
      logger.warning(String.format("%s does not exist.\n", annotationFile));
      annotationDoc = new BratDocument();
    }

    Path propagateFile = propagateBratDir.resolve(basename + ".ann");
    if (Files.exists(propagateFile)) {
      propagateDoc = BratIOUtils.read(Files.newBufferedReader(propagateFile), basename);
    } else {
      logger.warning(String.format("%s does not exist.\n", propagateFile));
      propagateDoc = new BratDocument();
    }

    Path negFile = negBratDir.resolve(basename + ".ann");
    if (Files.exists(negFile)) {
      negDoc = BratIOUtils.read(Files.newBufferedReader(negFile), basename);
    } else {
      logger.warning(String.format("%s does not exist.\n", negFile));
      negDoc = new BratDocument();
    }
  }

  @Override
  protected void processFile(String basename) throws Exception {
    logger.info("process " + basename);

    BratMerge merge = new BratMerge();

    String prefix = basename.substring(0, basename.indexOf('_'));

    // merge basic
    merge.addDocument(basicDoc);

    // merge technique
    merge.addDocument(techDoc);

    // merge annotation
    merge.addDocument(annotationDoc);

    // merge extended
    merge.addDocument(extendedDoc);

    // merge propagate
    merge.addDocument(propagateDoc);

    // merge neg
    merge.addDocument(negDoc);

    BratDocument doc = merge.getDoc();
    Path outputFile = outputDir.resolve(basename + ".ann");
    BratIOUtils.write(Files.newBufferedWriter(outputFile), doc);
  }

//  private boolean find(String prefix, BratEntity e1) throws IOException {
//    // basic
//    BratFileVisitor2 visitor = new BratFileVisitor2(prefix, e1);
//    Files.walkFileTree(basicBratDir, visitor);
//    Files.walkFileTree(techBratDir, visitor);
//
//    if (visitor.entities.isEmpty()) {
//      return false;
//    }
//
//    String line = null;
//    int start = 0;
//    for(String l: lines) {
//      int end = start + l.length();
//      if (start <= e1.totalSpan().lowerEndpoint() && e1.totalSpan().upperEndpoint() <= end) {
//        line = l;
//        break;
//      }
//      start = end + 1;
//    }
//
//    if (line == null) {
//      return false;
//    }
//
//    for(BratEntity ent: visitor.entities) {
//      if (line.contains(ent.getText())) {
//        return true;
//      }
//    }
//
//    return false;
//  }

//  private boolean find(String prefix, BratEntity e1, BratEntity e2) throws IOException {
//    // basic
//    BratFileVisitor visitor = new BratFileVisitor(prefix, e1, e2);
//    Files.walkFileTree(basicBratDir, visitor);
//    if (visitor.isFound) {
//      return true;
//    }
//
//    Files.walkFileTree(techBratDir, visitor);
//    if (visitor.isFound) {
//      return true;
//    }
//
//    return false;
//  }

//  static class BratFileVisitor2 extends SimpleFileVisitor<Path> {
//    String prefix;
//    BratEntity target1;
//    List<BratEntity> entities;
//
//    public BratFileVisitor2(String prefix, BratEntity target1) {
//      this.prefix = prefix;
//      this.target1 = target1;
//      entities = Lists.newArrayList();
//    }
//
//    @Override
//    public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) throws IOException {
//      String basename = FilenameUtils.getBaseName(file.toString());
//      if (basename.startsWith(prefix)) {
//        BratDocument bratDocument = BratIOUtils.read(Files.newBufferedReader(file), basename);
//        for(BratRelation rel: bratDocument.getRelations()) {
//          if (!rel.containsRole("Arg2")) {
//            continue;
//          }
//          BratEntity e1 = bratDocument.getEntity(rel.getArgId("Arg1"));
//          BratEntity e2 = bratDocument.getEntity(rel.getArgId("Arg2"));
//          if (e1.getText().equalsIgnoreCase(target1.getText())) {
//            entities.add(e2);
//          }
//          if (e2.getText().equalsIgnoreCase(target1.getText())) {
//            entities.add(e1);
//          }
//        }
//      }
//      return FileVisitResult.CONTINUE;
//    }
//  }

  static class BratFileVisitor extends SimpleFileVisitor<Path> {
    String prefix;
    BratEntity target1;
    BratEntity target2;
    boolean isFound;

    public BratFileVisitor(String prefix, BratEntity target1, BratEntity target2) {
      this.prefix = prefix;
      this.target1 = target1;
      this.target2 = target2;
    }

    @Override
    public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) throws IOException {
      String basename = FilenameUtils.getBaseName(file.toString());
      if (basename.startsWith(prefix)) {
        BratDocument bratDocument = BratIOUtils.read(Files.newBufferedReader(file), basename);
        for(BratRelation rel: bratDocument.getRelations()) {
          if (!rel.containsRole("Arg2")) {
            continue;
          }
          BratEntity e1 = bratDocument.getEntity(rel.getArgId("Arg1"));
          BratEntity e2 = bratDocument.getEntity(rel.getArgId("Arg2"));
          if ((e1.getText().equalsIgnoreCase(target1.getText())
              && e2.getText().equalsIgnoreCase(target2.getText()))
              || (e2.getText().equalsIgnoreCase(target1.getText())
              && e1.getText().equalsIgnoreCase(target2.getText()))) {
            isFound = true;
            return FileVisitResult.TERMINATE;
          }
        }
      }
      return FileVisitResult.CONTINUE;
    }
  }
}
