package com.pengyifan.biocreativev.cmd;

import com.google.common.collect.Lists;
import com.pengyifan.commons.nio.BatchProcessor;
import com.pengyifan.commons.nio.SimpleBasenameVisitor;
import com.pengyifan.kernel.MyProperties;
import com.pengyifan.kernel.allpathgraph.module.ApgConfiguration;
import com.pengyifan.kernel.allpathgraph.module.DefaultApgConfiguration;
import com.pengyifan.nlp.process.PtbAlignment;
import com.pengyifan.nlp.trees.PtbWriter;
import com.pengyifan.nlp.trees.TreebankReader;
import com.pengyifan.util.PropertiesLoader;
import edu.stanford.nlp.trees.Tree;
import edu.stanford.nlp.trees.Treebank;
import java.io.File;
import java.nio.file.DirectoryStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import java.util.StringJoiner;
import java.util.logging.LogManager;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;

/**
 * Used for bllip parser.
 */
public class PtbAlignmentClient extends SimpleBasenameVisitor {

  public static void main(String[] args)
      throws Exception {
    LogManager.getLogManager().readConfiguration(
        PropertiesLoader.loadLoggingPropertiesStream());


    MyProperties prop = new MyProperties("biocreative5");

    String dataset = "dataset2";
    Path root = Paths.get("tmp/biocreative5/dataset2"); //prop.getDropboxDir().resolve(dataset);

    List<String> list = Lists.newArrayList();
    try (DirectoryStream<Path> stream = Files.newDirectoryStream(root.resolve("brat"), "*.txt")) {
      for (Path file : stream) {
        String basename = FilenameUtils.getBaseName(file.toString());
        list.add(basename);
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
    list.sort(String::compareTo);

    PtbAlignmentClient s = new PtbAlignmentClient(new DefaultApgConfiguration());
    s.bratInputDir = root.resolve("brat");
    s.parsingInputDir = root.resolve("ptb1");
    s.parsingOutputDir = root.resolve("ptb2");
    BatchProcessor.walk(
        list,
        s
    );

  }

  //  private static final Logger logger = Logger.getAnonymousLogger();
  private final ApgConfiguration config;

  private Path bratInputDir;
  private Path parsingInputDir;
  private Path parsingOutputDir;

  private Treebank treebank;
  private String text;

  public PtbAlignmentClient(ApgConfiguration config) {
    this.config = config;
  }

  @Override
  public void preprocessFile(String basename)
      throws Exception {

    Path textFile = bratInputDir.resolve(basename + ".txt");
    text = new String(Files.readAllBytes(textFile));

    File ptbFile = parsingInputDir.resolve(basename + ".ptb").toFile();
    TreebankReader treebankReader = new TreebankReader(ptbFile,
        config.getTreeBuilder());
    try {
      treebank = treebankReader.read();
    } catch (Exception e) {

    }
    treebankReader.close();
  }

  @Override
  public final void processFile(String basename)
      throws Exception {
    try {
      new PtbAlignment(text, treebank).alignment();
    } catch (Exception e) {

    }
  }

  @Override
  public void postprocessFile(String basename)
      throws Exception {
    // output
    StringJoiner sj = new StringJoiner("\n");
    for (Tree tree : treebank) {
      sj.add(PtbWriter.print(tree));
    }
    Path ptbFile = parsingOutputDir.resolve(basename + ".ptb");
    FileUtils.write(ptbFile.toFile(), sj.toString());
  }
}