package com.pengyifan.biocreativev.cmd;

import com.pengyifan.brat.BratDocument;
import com.pengyifan.brat.io.BratIOUtils;
import com.pengyifan.commons.io.AbstractBatchProcessor;
import com.pengyifan.kernel.MyProperties;

import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Collection;
import java.util.logging.Logger;

public class Task1BratAdapter extends AbstractBatchProcessor {

  public static void main(String[] args) throws Exception {
    MyProperties prop = new MyProperties("biocreative5");
    String dataset = "dataset";

    Path dir = prop.getDropboxDir().resolve(dataset);

    Task1BratAdapter b = new Task1BratAdapter(BiocreativeUtils.getAllTextBasenames(dataset));
    b.annDir = dir.resolve("team312/brat");
    b.a1Dir = dir.resolve("team312/brat");
    b.process();
  }

  private static final Logger logger = Logger.getAnonymousLogger();
  private Path a1Dir;
  private Path annDir;

  public Task1BratAdapter(Collection<String> basenames) {
    super(basenames);
  }

  @Override
  protected void processFile(String basename) throws Exception {
    logger.info("process " + basename);

    Path annFile = annDir.resolve(basename + ".ann");
    if (!Files.exists(annFile)) {
      System.err.printf("%s does not exsist.\n", annFile);
      return;
    }

    BratDocument annDocument = BratIOUtils.read(Files.newBufferedReader(annFile), basename);

    // filter
    BratDocument a1Document = new BratDocument();
    annDocument.getEntities().stream()
        .filter(e -> e.getType().equals("Gene"))
        .forEach(e -> {
          e.setType("Protein");
          a1Document.addAnnotation(e);
        });

    Path a1File = a1Dir.resolve(basename + ".a1");
    BratIOUtils.write(Files.newBufferedWriter(a1File), a1Document);
  }
}
