package com.pengyifan.bionlpst.trigger;

import com.google.common.collect.ImmutableList;
import com.google.common.collect.Lists;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVParser;
import org.apache.commons.csv.CSVRecord;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;

import java.io.Closeable;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;
import java.util.Optional;

public class SimpleTriggerReader implements Closeable {

  private static final String TYPE_HEADER = "type";
  private static final String LEMMA_HEADER = "lemma";

  private static final CSVFormat FORMAT = CSVFormat.DEFAULT
      .withDelimiter('\t')
      .withCommentMarker('#')
      .withIgnoreEmptyLines(true)
      .withAllowMissingColumnNames(true)
      .withHeader(
          TYPE_HEADER,
          LEMMA_HEADER);

  private final CSVParser parser;

  public static List<Trigger> readAll(Path file)
      throws IOException {
    SimpleTriggerReader reader = new SimpleTriggerReader(file);
    List<Trigger> triggers = reader.readAll();
    reader.close();
    return triggers;
  }

  public SimpleTriggerReader(Path file)
      throws IOException {
    parser = FORMAT.parse(Files.newBufferedReader(file));
  }

  @Override
  public void close()
      throws IOException {
    parser.close();
  }

  public List<Trigger> readAll()
      throws IOException {
    List<Trigger> triggers = Lists.newArrayList();
    for (CSVRecord csvRecord : parser) {
      triggers.add(createTrigger(csvRecord));
    }
    return ImmutableList.copyOf(triggers);
  }

  private Trigger createTrigger(CSVRecord csvRecord) {
    return new SimpleTrigger()
        .setType(csvRecord.get(TYPE_HEADER))
        .setLemma(csvRecord.get(LEMMA_HEADER));
  }

  private static class SimpleTrigger extends Trigger {
    @Override
    public String getWord() {
      throw new UnsupportedOperationException("Only lemma is supported");
    }

    @Override
    public String getTag() {
      throw new UnsupportedOperationException("Only lemma is supported");
    }

    @Override
    public Optional<String> getMark() {
      throw new UnsupportedOperationException("Only lemma is supported");
    }

    @Override
    public Optional<Integer> getFrequency() {
      throw new UnsupportedOperationException("Only lemma is supported");
    }

    @Override
    public ImmutableList<String> getPatterns() {
      throw new UnsupportedOperationException("Only lemma is supported");
    }

    @Override
    public String toString() {
      return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE)
          .append("type", getType())
          .append("lemma", getLemma().orElse(null))
          .toString();
    }
  }
}
