package com.pengyifan.bionlpst.trigger;

import java.util.ArrayList;
import java.util.Objects;
import java.util.Optional;

import static com.google.common.base.Preconditions.checkNotNull;

import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;

import com.google.common.collect.ImmutableList;
import com.pengyifan.commons.collections.dependencygraph.annotations.FrequencyAnnotation;
import com.pengyifan.commons.collections.dependencygraph.annotations.PatternsAnnotation;

import edu.stanford.nlp.ling.CoreAnnotations;
import edu.stanford.nlp.ling.CoreLabel;

public class Trigger {

  private final CoreLabel label;

  public Trigger() {
    label = new CoreLabel();
    label.set(PatternsAnnotation.class, new ArrayList<String>());
  }

  public String getType() {
    checkNotNull(
        label.get(CoreAnnotations.EntityTypeAnnotation.class),
        "Type is not set");
    return label.get(CoreAnnotations.EntityTypeAnnotation.class);
  }

  Trigger setType(final String type) {
    label.set(CoreAnnotations.EntityTypeAnnotation.class, type);
    return this;
  }

  Trigger addPattern(final String pattern) {
    label.get(PatternsAnnotation.class).add(pattern);
    return this;
  }

  public ImmutableList<String> getPatterns() {
    return ImmutableList.copyOf(label.get(PatternsAnnotation.class));
  }

  public Optional<Integer> getFrequency() {
    return Optional.ofNullable(label.get(FrequencyAnnotation.class));
  }

  Trigger setFrequency(int freq) {
    label.set(FrequencyAnnotation.class, freq);
    return this;
  }

  public Optional<String> getMark() {
    return Optional.ofNullable(label
        .get(CoreAnnotations.MarkingAnnotation.class));
  }

  public String getWord() {
    checkNotNull(label.word(), "Word is not set");
    return label.word();
  }

  public String getTag() {
    checkNotNull(label.tag(), "Tag is not set");
    return label.tag();
  }

  public Optional<String> getLemma() {
    return Optional.ofNullable(label.lemma());
  }

  Trigger setWord(String word) {
    label.setWord(word);
    return this;
  }

  Trigger setTag(String tag) {
    label.setTag(tag);
    return this;
  }

  Trigger setLemma(String lemma) {
    label.setLemma(lemma);
    return this;
  }

  Trigger setMark(String mark) {
    label.set(CoreAnnotations.MarkingAnnotation.class, mark);
    return this;
  }

  @Override
  public String toString() {
    return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE)
        .append("type", getType())
        .append("word", getWord())
        .append("tag", getTag())
        .append("lemma", getLemma().orElse(null))
        .append("freq", getFrequency().orElse(null))
        .append("mark", getMark().orElse(null))
        .append("patterns", getPatterns())
        .toString();
  }

  @Override
  public int hashCode() {
    return Objects.hash(
        getWord(),
        getTag(),
        getFrequency(),
        getMark(),
        getPatterns());
  }

  @Override
  public boolean equals(Object obj) {
    if (this == obj) {
      return true;
    }
    if (obj == null || getClass() != obj.getClass()) {
      return false;
    }
    Trigger other = (Trigger) obj;
    return Objects.equals(getWord(), other.getWord())
        && Objects.equals(getType(), other.getType())
        && Objects.equals(getTag(), other.getTag())
        && Objects.equals(getFrequency(), other.getFrequency())
        && Objects.equals(getMark(), other.getMark())
        && Objects.equals(getPatterns(), other.getPatterns());
  }
}
