package com.pengyifan.bionlpst.utils;

import java.io.Closeable;
import java.io.IOException;
import java.io.PrintStream;
import java.text.DecimalFormat;

import com.google.common.base.Strings;

import edu.stanford.nlp.stats.PrecisionRecallStats;

public class ResultPrinter implements Closeable {

  private static final String FORMAT = "%-18s %7s (%5s) %7s (%5s) %7s %7s %7s";
  private static final String HEADER = String.format(
      FORMAT, "Event Class", "gold", "match", "answer", "match", "recall",
      "prec.", "fscore");
  private static final String HEADER_LINE = Strings
      .repeat("-", HEADER.length());

  private static final DecimalFormat PERCENTAGE_FORMAT = new DecimalFormat(
      "00.00");
  private static final DecimalFormat INTEGER_FORMAT = new DecimalFormat(
      "#,###");
  private final PrintStream display;

  public ResultPrinter() {
    this(System.out);
  }

  public ResultPrinter(PrintStream ps) {
    display = ps;
  }

  public void printTitle() {
    display.println(HEADER);
    display.println(HEADER_LINE);
  }
  
  public void printRow(String type, PrecisionRecallStats stats) {
    String row = String.format(
        FORMAT,
        type,
        getInteger(stats.getTP() + stats.getFN()),
        getInteger(stats.getTP()),
        getInteger(stats.getTP() + stats.getFP()),
        getInteger(stats.getTP()),
        getPercentage(stats.getRecall() * 100),
        getPercentage(stats.getPrecision() * 100),
        getPercentage(stats.getFMeasure() * 100));
    display.println(row);
  }

//  public void printRow(String type, int tp, int fp, int fn) {
//    printRow(type, new PrecisionRecallStats(tp, fp, fn));
//  }

  private String getInteger(int integer) {
    return INTEGER_FORMAT.format(integer);
  }

  private String getPercentage(double percentage) {
    if (Double.isNaN(percentage)) {
      return "--";
    } else {
      return PERCENTAGE_FORMAT.format(percentage);
    }
  }

  @Override
  public void close()
      throws IOException {
    display.close();
  }
}
