package com.pengyifan.commons.collections.dependencygraph;

import static com.google.common.base.Preconditions.checkNotNull;

import java.util.Optional;
import java.util.function.Predicate;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

import com.google.common.collect.Iterables;
import com.pengyifan.commons.collections.MyIterables;

public class Dependency {

  public static Predicate<Dependency> ofPrepPredicate(
      @Nonnull final String first,
      @Nonnull final String... rest) {
    return e -> e.equals(DependencyName.PREP, first, rest)
        || e.equals(DependencyName.PREPC, first, rest);
  }

  public static final Predicate<Dependency> NnPredicate =
      e -> e.getName().equals(DependencyName.NN, DependencyName.AMOD);

  public static Predicate<Dependency> ofPredicate(
      @Nonnull final DependencyName first,
      @Nonnull final DependencyName... rest) {
    return d -> d.getName().equals(first, rest);
  }

  public static Predicate<Dependency> ofPredicate(
      @Nonnull final DependencyName name,
      @Nonnull final String first,
      @Nonnull final String... rest) {
    return d -> d.equals(name, first, rest);
  }

  private DependencyName name;
  private Optional<String> specific;

  public Dependency(@Nonnull DependencyName name, @Nullable String specific) {
    this.name = name;
    this.specific = Optional.ofNullable(specific);
  }

  @Nonnull
  public DependencyName getName() {
    return name;
  }

  public void setName(@Nonnull DependencyName name) {
    this.name = name;
  }

  @Nonnull
  public Optional<String> getSpecific() {
    return specific;
  }

  public void setSpecific(@Nullable String specific) {
    this.specific = Optional.ofNullable(specific);
  }

  public boolean equals(@Nonnull DependencyName name, @Nonnull String first, @Nonnull String... rest) {
    checkNotNull(name);
    checkNotNull(first);
    return this.name == name
        && this.specific.isPresent()
        && Iterables.contains(MyIterables.iterable(first, rest), specific.get());
  }
}
