package com.pengyifan.commons.collections.dependencygraph;

import static com.google.common.base.Preconditions.checkNotNull;

import java.util.Map;

import javax.annotation.Nonnull;

import com.pengyifan.commons.lang.FromStringBuilder;
import org.apache.commons.lang3.builder.ToStringBuilder;

import com.google.common.annotations.VisibleForTesting;
import com.pengyifan.commons.collections.dependencygraph.annotations.DependencyAnnotation;
import com.pengyifan.commons.collections.dependencygraph.annotations.PositionAnnotation;
import com.pengyifan.commons.collections.dependencygraph.annotations.ShortestPathDirectionAnnotation;
import com.pengyifan.commons.collections.indexgraph.IndexObject;

public class DependencyGraphEdge extends IndexObject {

  public static DependencyGraphEdge parse(@Nonnull String s) {
    // DependencyGraphEdge[index=29,name=prep,specific=of,position=IN_SHORTEST_PATH,neg=false,weight=0.95]
    Map<String, String> map = FromStringBuilder.stringToMap(s);
    Dependency dependency = new Dependency(
        DependencyName.valueOf(map.get("name")),
        map.containsKey("specific") ? map.get("specific") : null);
    return new DependencyGraphEdge(
        Integer.parseInt(map.get("index")),
        dependency,
        Position.valueOf(map.get("position")),
        ShortestPathDirection.valueOf(map.get("spdirection")));
  }

  public static DependencyGraphEdge of(int index,
      @Nonnull Dependency dependency) {
    DependencyGraphEdge edge = new DependencyGraphEdge(index);
    edge.setDependency(dependency);
    edge.setPosition(Position.UNKNOWN);
    edge.setShortestPathDirection(ShortestPathDirection.UNKNOWN);
    return edge;
  }

  @VisibleForTesting
  DependencyGraphEdge(int index) {
    super(index);
  }

  public DependencyGraphEdge(int index,
      @Nonnull DependencyGraphEdge edge) {
    this(index,
        edge.getDependency(),
        edge.getPositionLabel(),
        edge.getShortestPathDirection());
  }

  public DependencyGraphEdge(
      int index,
      @Nonnull Dependency dependency,
      @Nonnull Position position,
      @Nonnull ShortestPathDirection direction) {
    super(index);
    setDependency(dependency);
    setPosition(position);
    setShortestPathDirection(direction);
  }

  @Nonnull
  public Dependency getDependency() {
    return checkNotNull(
        label.get(DependencyAnnotation.class),
        "Dependency is not set");
  }

  @Nonnull
  public Position getPositionLabel() {
    return checkNotNull(
        label.get(PositionAnnotation.class),
        "Position is not set: %s",
        new ToStringBuilder(this, new MyStyle())
            .append("index", getIndex())
            .append("name", getDependency().getName())
            .append("specific", getDependency().getSpecific().orElse(null))
            .toString());
  }

  @Nonnull
  public ShortestPathDirection getShortestPathDirection() {
    return checkNotNull(
        label.get(ShortestPathDirectionAnnotation.class),
        "Shortest path direction is not set");
  }

  private void setDependency(@Nonnull Dependency dependency) {
    label.set(DependencyAnnotation.class,
        checkNotNull(dependency, "Dependency is null"));
  }

  public void setShortestPathDirection(
      @Nonnull ShortestPathDirection direction) {
    label.set(ShortestPathDirectionAnnotation.class,
        checkNotNull(direction, "Shortest path direction is null"));
  }

  public void setPosition(@Nonnull Position positionLabel) {
    checkNotNull(positionLabel, "Position is null");
    label.set(PositionAnnotation.class, positionLabel);
  }

  @Override
  public boolean equals(Object obj) {
    if (!(obj instanceof DependencyGraphEdge)) {
      throw new UnsupportedOperationException("Should not use equals");
    }
    return super.equals(obj);
  }

  @Override
  public String toString() {
    return new ToStringBuilder(this, new MyStyle())
        .append("index", getIndex())
        .append("name", getDependency().getName())
        .append("specific", getDependency().getSpecific().orElse(null))
        .append("position", getPositionLabel())
        .append("spdirection", getShortestPathDirection())
        .toString();
  }

}
