package com.pengyifan.commons.collections.dependencygraph;

import javax.annotation.Nonnull;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.function.Predicate;

public class DependencyGraphMatcher {

  private final DependencyGraph graph;
  private final Map<Integer, Predicate<DependencyGraphVertex>> predicateMap;
  private final Iterator<Map<Predicate<DependencyGraphVertex>, DependencyGraphVertex>> itr;
  private Map<Predicate<DependencyGraphVertex>, DependencyGraphVertex> current;

  DependencyGraphMatcher(
      @Nonnull DependencyGraph graph,
      @Nonnull List<Map<Predicate<DependencyGraphVertex>, DependencyGraphVertex>> list,
      @Nonnull Map<Integer, Predicate<DependencyGraphVertex>> predicateMap) {
    this.graph = graph;
    this.itr = list.iterator();
    this.predicateMap = predicateMap;
  }

  public DependencyGraph getGraph() {
    return graph;
  }
  
  public boolean find() {
    boolean hasNext = itr.hasNext();
    if (hasNext) {
      current = itr.next();
    } else {
      current = null;
    }
    return hasNext;
  }
  
  public DependencyGraphVertex get(int index) {
    if (current == null) {
      throw new IllegalStateException("No match found");
    }
    if (!predicateMap.containsKey(index)) {
      throw new IndexOutOfBoundsException("No index " + index);
    }
    return current.get(predicateMap.get(index));
  }
}