package com.pengyifan.commons.collections.dependencygraph;

import static com.google.common.base.Preconditions.checkArgument;

import java.util.Map;
import java.util.function.Predicate;

import org.jgrapht.DirectedGraph;
import org.jgrapht.EdgeFactory;
import org.jgrapht.graph.DefaultDirectedGraph;

import com.google.common.collect.Maps;
import com.pengyifan.commons.collections.indexgraph.ExactSubgraphMatchingPredicate;

public class DependencyGraphPattern {

  private static class PredicateEdgeFactory
      implements
      EdgeFactory<Predicate<DependencyGraphVertex>, Predicate<DependencyGraphEdge>> {

    @Override
    public Predicate<DependencyGraphEdge> createEdge(
        Predicate<DependencyGraphVertex> sourceVertex,
        Predicate<DependencyGraphVertex> targetVertex) {
      return null;
    }
  }

  private DirectedGraph<Predicate<DependencyGraphVertex>, Predicate<DependencyGraphEdge>> graph;
  private Map<Integer, Predicate<DependencyGraphVertex>> map;
  
  public static DependencyGraphPattern of() {
    return new DependencyGraphPattern();
  }

  private DependencyGraphPattern() {
    graph = new DefaultDirectedGraph<>(new PredicateEdgeFactory());
    map = Maps.newHashMap();
  }

  public DependencyGraphPattern addVertex(int index, Predicate<DependencyGraphVertex> vP) {
    checkArgument(!map.containsKey(index), "Duplicated index: %s", index);
    map.put(index, vP);
    graph.addVertex(vP);
    return this;
  }

  public DependencyGraphPattern addEdge(int source, int target, 
      Predicate<Dependency> dP) {
    Predicate<DependencyGraphVertex> sP = map.get(source);
    Predicate<DependencyGraphVertex> tP = map.get(target);
    graph.addEdge(sP, tP, e->dP.test(e.getDependency()));
    return this;
  }

  public DependencyGraphMatcher matcher(DependencyGraph graph) {
    ExactSubgraphMatchingPredicate<DependencyGraphVertex, DependencyGraphEdge> esm =
        new ExactSubgraphMatchingPredicate<>(this.graph, graph);
    return new DependencyGraphMatcher(graph, esm.getSubgraphMatchingMatches(), map);
  }
}