package com.pengyifan.commons.collections.dependencygraph;

import static com.google.common.base.Preconditions.checkNotNull;

import java.util.Map;

import com.pengyifan.commons.lang.FromStringBuilder;
import org.apache.commons.lang3.builder.ToStringBuilder;

import com.google.common.annotations.VisibleForTesting;
import com.pengyifan.commons.collections.dependencygraph.annotations.NamedEntityAnnotation;
import com.pengyifan.commons.collections.dependencygraph.annotations.PartOfSpeechAnnotation;
import com.pengyifan.commons.collections.dependencygraph.annotations.PositionAnnotation;
import com.pengyifan.commons.collections.indexgraph.IndexObject;

import edu.stanford.nlp.ling.CoreAnnotations.CharacterOffsetBeginAnnotation;
import edu.stanford.nlp.ling.CoreAnnotations.CharacterOffsetEndAnnotation;
import edu.stanford.nlp.ling.CoreAnnotations.LemmaAnnotation;
import edu.stanford.nlp.ling.CoreAnnotations.TextAnnotation;

public class DependencyGraphVertex extends IndexObject {

  @VisibleForTesting
  DependencyGraphVertex(int index) {
    super(index);
  }

  public DependencyGraphVertex(
      int index,
      String word,
      PartOfSpeech tag,
      String lemma,
      NamedEntity ner,
      Position position,
      int beginPosition,
      int endPosition) {
    super(index);
    setWord(word);
    setTag(tag);
    setLemma(lemma);
    setNer(ner);
    setPosition(position);
    setBeginPosition(beginPosition);
    setEndPosition(endPosition);
  }

  public Integer getBeginPosition() {
    return label.beginPosition();
  }

  public Integer getEndPosition() {
    return label.endPosition();
  }

  public NamedEntity getNer() {
    checkNotNull(
        label.get(NamedEntityAnnotation.class),
        "Named entity is not set");
    return label.get(NamedEntityAnnotation.class);
  }

  public String getLemma() {
    checkNotNull(
        label.lemma(),
        "Lemma is not set");
    return label.lemma();
  }

  public Position getPosition() {
    checkNotNull(
        label.get(PositionAnnotation.class),
        "Position is not set");
    return label.get(PositionAnnotation.class);
  }

  public PartOfSpeech getTag() {
    checkNotNull(label.get(PartOfSpeechAnnotation.class), "Tag is not set");
    return label.get(PartOfSpeechAnnotation.class);
  }

  public String getWord() {
    checkNotNull(label.word(), "Word is not set");
    return label.word();
  }

  public void setBeginPosition(int beginPos) {
    label.setBeginPosition(beginPos);
  }

  public void setEndPosition(int endPos) {
    label.setEndPosition(endPos);
  }

  public void setNer(NamedEntity ner) {
    label.set(NamedEntityAnnotation.class, ner);
  }

  public void setLemma(String lemma) {
    label.setLemma(lemma);
  }

  public void setPosition(Position position) {
    label.set(PositionAnnotation.class, position);
  }

  public void setTag(PartOfSpeech tag) {
    label.set(PartOfSpeechAnnotation.class, tag);
  }

  public void setWord(String word) {
    label.setWord(word);
  }

  @Override
  public String toString() {
    return new ToStringBuilder(this, new MyStyle())
        .append("index", getIndex())
        .append("begin", label.get(CharacterOffsetBeginAnnotation.class))
        .append("end", label.get(CharacterOffsetEndAnnotation.class))
        .append("word", label.get(TextAnnotation.class))
        .append("tag", label.get(PartOfSpeechAnnotation.class))
        .append("lemma", label.get(LemmaAnnotation.class))
        .append("ner", label.get(NamedEntityAnnotation.class))
        .append("position", label.get(PositionAnnotation.class))
        .toString();
  }

  public static DependencyGraphVertex parse(String s) {
    Map<String, String> map = FromStringBuilder.stringToMap(s);
    return new DependencyGraphVertex(
        Integer.parseInt(map.get("index")),
        map.get("word"),
        map.containsKey("tag") ? PartOfSpeech.valueOf(map.get("tag")) : null,
        map.containsKey("lemma") ? map.get("lemma") : null,
        map.containsKey("ner") ?
            NamedEntity.valueOf(map.get("ner")) : null,
        map.containsKey("position") ?
            Position.valueOf(map.get("position")) : null,
        Integer.parseInt(map.get("begin")),
        Integer.parseInt(map.get("end")));
  }

  @Override
  public boolean equals(Object obj) {
    if (!(obj instanceof DependencyGraphVertex)) {
      throw new UnsupportedOperationException("Should not use equals");
    }
    return super.equals(obj);
  }
}
