package com.pengyifan.commons.collections.dependencygraph;

import javax.annotation.Nonnull;

import com.google.common.collect.Iterables;
import com.pengyifan.commons.collections.MyIterables;

public enum DependencyName {
  UNKNOWN, /*NOT_DEP, NOT_SEM,*/

  // syntactic
  ACOMP, ADVCL, ADVMOD, AGENT, AMOD, APPOS, AUX, AUXPASS, CC,
  CCOMP, CONJ, COP, CSUBJ, CSUBJPASS, DEP, DET, DISCOURSE, DOBJ, EXPL,
  IOBJ, MARK, MWE, NEG, NN, NPADVMOD, NSUBJ, NSUBJPASS,
  NUM, NUMBER, PARATAXIS, PCOMP, POBJ, POSS, POSSESSIVE, PRECONJ, PREDET, PREP,
  PREPC, PRT, PUNCT, QUANTMOD, RCMOD, REF, ROOT, TMOD, VMOD, XCOMP, XSUBJ,
  
  // seq
  SEQ,

  // referential
  IS_A, MEMBER_COLLECTION, PART_WHOLE, /* NULL_ARG, */
  COREF, DESCRIPTIVE,

  // predicate argument relation
  ARG0, ARG1;

  public boolean equals(
      @Nonnull DependencyName first,
      @Nonnull DependencyName... rest) {
    return Iterables.contains(MyIterables.iterable(first, rest), this);
  }

  public boolean isSyntactic() {
    switch (this) {
    case ACOMP:
    case ADVCL:
    case ADVMOD:
    case AGENT:
    case AMOD:
    case APPOS:
    case AUX:
    case AUXPASS:
    case CC:
    case CCOMP:
    case CONJ:
    case COP:
    case CSUBJ:
    case CSUBJPASS:
    case DEP:
    case DET:
    case DISCOURSE:
    case DOBJ:
    case EXPL:
    case IOBJ:
    case MARK:
    case MWE:
    case NEG:
    case NN:
    case NPADVMOD:
    case NSUBJ:
    case NSUBJPASS:
    case NUM:
    case NUMBER:
    case PARATAXIS:
    case PCOMP:
    case POBJ:
    case POSS:
    case POSSESSIVE:
    case PRECONJ:
    case PREDET:
    case PREP:
    case PREPC:
    case PRT:
    case PUNCT:
    case QUANTMOD:
    case RCMOD:
    case REF:
    case ROOT:
    case TMOD:
    case VMOD:
    case XCOMP:
    case XSUBJ:
      return true;
    default:
      return false;
    }
  }
  
  public boolean isSemantic() {
    return isArgument() || isReferentialLink();
  }

  public boolean isArgument() {
    switch (this) {
    case ARG0:
    case ARG1:
      return true;
    default:
      return false;
    }
  }

  public boolean isReferentialLink() {
    switch (this) {
    case IS_A:
    case MEMBER_COLLECTION:
    case PART_WHOLE:
    case COREF:
    case DESCRIPTIVE:
      return true;
    default:
      return false;
    }
  }
}
