package com.pengyifan.commons.collections.dependencygraph;

import javax.annotation.Nonnull;

import com.google.common.collect.Iterables;
import com.pengyifan.commons.collections.MyIterables;
import com.pengyifan.nlp.ling.verb.VerbVoice;

public enum PartOfSpeech {
  UNKNOWN, RRB, CC, CD, COLON, COMMA, DT, EX, FW, IN, JJ, JJR, JJS, LS, LRB, MD, NN, NNP, NNPS, NNS, PDT,
  POS, PRP, PRP$, RB, RBS, RBR, RP, STOP, TO, VB, VBD, VBG, VBN, VBP, VBZ, WDT, WRB, WP, WP$;

  public VerbVoice getVerbVoice() {
    switch (this) {
    case VB:
    case VBZ:
    case VBD:
    case VBP:
    case VBG:
      return VerbVoice.ACTIVE;
    case VBN:
      return VerbVoice.PASSIVE;
    default:
      return VerbVoice.UNKNOWN;
    }
  }

  public boolean isVerb() {
    switch (this) {
    case VB:
    case VBZ:
    case VBD:
    case VBP:
    case VBG:
    case VBN:
      return true;
    default:
      return false;
    }
  }

  public boolean isNoun() {
    return this == NN || this == NNS || this == NNP || this == NNPS;
  }

  public static PartOfSpeech getEnum(String s) {
    switch (s) {
    case ":":
      return COLON;
    case ",":
      return COMMA;
    case ".":
      return STOP;
    case "-LRB-":
      return LRB;
    case "-RRB-":
      return RRB;
    default:
      return valueOf(s);
    }
  }

  public boolean isActiveVerb() {
    return getVerbVoice() == VerbVoice.ACTIVE;
  }

  public boolean isPassiveVerb() {
    return getVerbVoice() == VerbVoice.PASSIVE;
  }

  public boolean equals(
      @Nonnull PartOfSpeech first,
      @Nonnull PartOfSpeech... rest) {
    return Iterables.contains(MyIterables.iterable(first, rest), this);
  }
}
