package com.pengyifan.commons.convert;

import com.pengyifan.bioc.BioCAnnotation;
import com.pengyifan.bioc.BioCLocation;
import com.pengyifan.bioc.BioCNode;
import com.pengyifan.bioc.BioCRelation;
import com.pengyifan.brat.BratEntity;
import com.pengyifan.brat.BratRelation;

import static com.google.common.base.Preconditions.checkArgument;

public class BiocToBratConverter {

  public BratEntity convert(BioCAnnotation biocAnnotation) {
    BratEntity bratEntity = new BratEntity();
    bratEntity.setId(biocAnnotation.getID());
    checkArgument(
        biocAnnotation.getText().isPresent(),
        "BioC annotation has no text");
    bratEntity.setText(biocAnnotation.getText().get());
    checkArgument(
        biocAnnotation.getInfon("type").isPresent(),
        "BioC annotation has no type information");
    bratEntity.setType(biocAnnotation.getInfon("type").get());
    for (BioCLocation loc : biocAnnotation.getLocations()) {
      bratEntity.addSpan(loc.getOffset(), loc.getOffset() + loc.getLength());
    }
    return bratEntity;
  }

  public BratRelation convert(BioCRelation biocRelation) {
    BratRelation bratRelation = new BratRelation();
    bratRelation.setId(biocRelation.getID());
    bratRelation.setType(biocRelation.getInfon("relation type").get());
    for (BioCNode node : biocRelation.getNodes()) {
      bratRelation.putArgument(node.getRole(), node.getRefid());
    }
    return bratRelation;
  }
}