package com.pengyifan.commons.convert;

import com.google.common.collect.Range;
import com.pengyifan.bioc.BioCAnnotation;
import com.pengyifan.bioc.BioCDocument;
import com.pengyifan.bioc.BioCLocation;
import com.pengyifan.bioc.BioCNode;
import com.pengyifan.bioc.BioCPassage;
import com.pengyifan.bioc.BioCRelation;
import com.pengyifan.brat.BratDocument;
import com.pengyifan.brat.BratEntity;
import com.pengyifan.brat.BratEvent;
import com.pengyifan.brat.BratRelation;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;


public class BratToBiocConverter {
  
  public BioCDocument translate(BratDocument bratDocument) {
    BioCDocument biocDocument = new BioCDocument();
    biocDocument.setID(bratDocument.getDocId());
    
    BioCPassage biocPassage = new BioCPassage();
    biocPassage.setText(bratDocument.getText());
    biocPassage.setOffset(0);
    biocDocument.addPassage(biocPassage);
    
    // entity
    for(BratEntity bratEntity: bratDocument.getEntities()) {
      biocPassage.addAnnotation(convert(bratEntity));
    }
    // relation
    for(BratRelation bratRelation: bratDocument.getRelations()) {
      biocPassage.addRelation(convert(bratRelation));
    }
    // event
    for(BratEvent bratEvent: bratDocument.getEvents()) {
      biocPassage.addRelation(convert(bratEvent));
    }
    return biocDocument;
  }
  
  public BioCRelation convert(BratRelation bratRelation) {
    BioCRelation biocRel = new BioCRelation();
    biocRel.setID(bratRelation.getId());
    Map<String, String> infons = new HashMap<String, String>();
    infons.put("type", bratRelation.getType());
    biocRel.setInfons(infons);
    // arg
    for (Entry<String, String> pair : bratRelation.getArguments().entrySet()) {
      biocRel.addNode(new BioCNode(pair.getValue(), pair.getKey()));
    }
    return biocRel;
  }

  public BioCRelation convert(BratEvent bratEvent) {
    BioCRelation biocRelation = new BioCRelation();
    biocRelation.setID(bratEvent.getId());
    Map<String, String> infons = new HashMap<String, String>();
    infons.put("type", bratEvent.getType());
    biocRelation.setInfons(infons);
    // trigger
    biocRelation.addNode(new BioCNode(bratEvent.getTriggerId(), bratEvent.getType()));
    // arg
    for (Entry<String, String> pair : bratEvent.getArguments().entrySet()) {
      biocRelation.addNode(new BioCNode(pair.getValue(), pair.getKey()));
    }
    return biocRelation;
  }

  public BioCAnnotation convert(BratEntity bratEntity) {
    BioCAnnotation biocAnnotatioin = new BioCAnnotation();
    biocAnnotatioin.setID(bratEntity.getId());
    biocAnnotatioin.setText(bratEntity.getText());
    for (Range<Integer> range : bratEntity.getSpans().asRanges()) {
      biocAnnotatioin.addLocation(new BioCLocation(range.lowerEndpoint(), range
          .upperEndpoint() - range.lowerEndpoint()));
    }
    Map<String, String> infons = new HashMap<String, String>();
    infons.put("type", bratEntity.getType());
    biocAnnotatioin.setInfons(infons);
    return biocAnnotatioin;
  }
}
