package com.pengyifan.experimental;

import com.pengyifan.bioc.BioCCollection;
import com.pengyifan.bioc.BioCDocument;
import com.pengyifan.bioc.BioCSentence;
import com.pengyifan.bioc.io.BioCCollectionWriter;
import com.pengyifan.brat.BratDocument;
import com.pengyifan.brat.io.BratIOUtils;
import com.pengyifan.commons.convert.BratToBiocConverter;
import com.pengyifan.commons.io.AbstractBatchProcessor;
import com.pengyifan.commons.io.BasenameUtils;
import com.pengyifan.kernel.MyProperties;
import com.pengyifan.kernel.allpathgraph.module.DefaultApgConfiguration;
import edu.stanford.nlp.trees.MemoryTreebank;
import org.apache.commons.io.FileUtils;

import java.io.StringReader;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Collection;

/**
 * Created by ypeng on 10/6/15.
 */
public class AIMedBrat2BioC extends AbstractBatchProcessor {
  public static void main(String[] args) throws Exception {
    MyProperties prop = new MyProperties("aimed");
    String dataset = "dataset";
    Path root = prop.getTempDir().resolve(dataset);

    AIMedBrat2BioC s = new AIMedBrat2BioC(
        BasenameUtils.getTextBasenames(
            prop.getDropboxDir().resolve("whole-byDocument/preprocessed/brat")
        ));

    s.bratInputDir = prop.getDropboxDir().resolve("whole-byDocument/preprocessed/brat");
    s.outputFile = root.resolve("aimed.xml");
    s.process();
  }

  BioCCollection collection;
  BratToBiocConverter converter;
  Path bratInputDir;
  Path outputFile;
  BratDocument bratDocument;

  public AIMedBrat2BioC(Collection<String> basenames) {
    super(basenames);
    converter = new BratToBiocConverter();
  }

  @Override
  protected void preprocess() throws Exception {
    collection = new BioCCollection();
    collection.setSource("AIMed");
  }

  @Override
  protected void preprocessFile(String basename) throws Exception {

    System.out.println(basename);

    Path textFile = bratInputDir.resolve(basename + ".txt");
    String text = new String(Files.readAllBytes(textFile));

    Path a1File = bratInputDir.resolve(basename + ".a1");
    String s = FileUtils.readFileToString(a1File.toFile());

    Path a2File = bratInputDir.resolve(basename + ".a2");
    s += "\n" + FileUtils.readFileToString(a2File.toFile());

    bratDocument = BratIOUtils.read(new StringReader(s), basename);
    bratDocument.setText(text);
    bratDocument.setDocId(basename);
  }

  @Override
  protected void postprocess() throws Exception {
    BioCCollectionWriter writer = new BioCCollectionWriter(outputFile.toFile());
    writer.writeCollection(collection);
    writer.close();
  }

  @Override
  protected void processFile(String basename) throws Exception {
    BioCDocument bioCDocument = converter.translate(bratDocument);
    collection.addDocument(bioCDocument);
  }
}
