package com.pengyifan.experimental;

import com.pengyifan.bioc.BioCCollection;
import com.pengyifan.bioc.BioCDocument;
import com.pengyifan.bioc.BioCPassage;
import com.pengyifan.bioc.io.BioCCollectionReader;
import com.pengyifan.bioc.util.BioCPassageIterator;
import com.pengyifan.bioc.util.BioCUtils;
import com.pengyifan.brat.BratDocument;
import com.pengyifan.brat.io.BratIOUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;

import javax.xml.stream.XMLStreamException;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;

public class BioC2Brat {

  public static void main(String[] args) throws IOException, XMLStreamException {
    new BioC2Brat().doMain(args);
  }

  public void doMain(String [] args) throws IOException, XMLStreamException {
    Path idir = Paths.get("tmp/biocreative5/dataset/20-document/bioc-origin/");
    Path odir = Paths.get("tmp/biocreative5/dataset/20-document/brat");
    Path odir2 = Paths.get("tmp/biocreative5/dataset/20-document/bioc-shift");

    for (Path p : Files.newDirectoryStream(idir, "*.xml")) {
      System.out.println(p);

      BioCCollection c = BioCUtils.readCollection(p);

      StringBuilder sb = new StringBuilder();
      for (BioCDocument document : c.getDocuments()) {
        for (BioCPassage passage : document.getPassages()) {
          fillText(sb, passage.getOffset());
          int newOffset = sb.length();
          if (newOffset > passage.getOffset()) {
            sb.append('\n');
            newOffset++;
            System.err.printf("Shift %d chars\n", newOffset);
          }
          if (passage.getText().isPresent()) {
            sb.append(passage.getText().get());
          } else {
            System.err.println("Cannot find text in passage: " + passage.getOffset());
          }
          passage.putInfon("old-offset", Integer.toString(passage.getOffset()));
          passage.setOffset(newOffset);
        }
      }

      Path o = odir.resolve(FilenameUtils.getBaseName(p.getFileName().toString()) + ".txt");
      FileUtils.write(o.toFile(), sb);

      Path o2 = odir.resolve(FilenameUtils.getBaseName(p.getFileName().toString()) + ".ann");
      FileUtils.write(o2.toFile(), "");

      Path o3 = odir2.resolve(FilenameUtils.getBaseName(p.getFileName().toString()) + ".xml");
      BioCUtils.writeCollectioin(o3, c);
    }
  }

  private StringBuilder fillText(StringBuilder sb, int offset) {
    while (sb.length() < offset) {
      sb.append('\n');
    }
    return sb;
  }
}
