package com.pengyifan.experimental;

import com.google.common.base.Joiner;
import com.google.common.base.Splitter;
import com.google.common.collect.HashMultiset;
import com.google.common.collect.Lists;
import com.google.common.collect.Multiset;
import com.google.common.collect.Multisets;
import com.google.common.collect.Sets;
import com.pengyifan.biocreativev.PassageType;
import com.pengyifan.biocreativev.cmd.Documents20;
import com.pengyifan.nlp.ling.string.EpitopeString;
import edu.stanford.nlp.stats.PrecisionRecallStats;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.LineNumberReader;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.StringJoiner;
import java.util.stream.Collectors;

import static com.google.common.base.Preconditions.checkNotNull;

public class BioGridEval {
  public static void main(String[] args) throws IOException {

    // gold
    List<Gold> golds = Lists.newArrayList(); //Sets.newTreeSet();
    // file
    Set<String> files = Sets.newHashSet();

    Path goldFile = Paths.get(System.getProperty("user.home") +
        "/Dropbox/Researches/data/biocreative5/dataset/gold4-20.csv");
    LineNumberReader reader = new LineNumberReader(Files.newBufferedReader(goldFile));
    reader.readLine();
    String line;
    while ((line = reader.readLine()) != null) {
      Gold g = new Gold();
      String[] toks = line.split("\t");
      g.docId = toks[0];

      if (!Documents20.contains(g.docId)) {
        continue;
      }

      try {
        g.arg1ID = Integer.parseInt(toks[1]);
      } catch (NumberFormatException e) {
        g.arg1ID = -1;
      }

      try {
        g.arg2ID = Integer.parseInt(toks[2]);
      } catch (NumberFormatException e) {
        g.arg2ID = -1;
      }
      g.arg1s.add(toks[3]);
      g.arg1s.addAll(Lists.newArrayList(toks[5].split("[|]")));
      g.arg2s.add(toks[4]);
      g.arg2s.addAll(Lists.newArrayList(toks[6].split("[|]")));
      //      if (Integer.compare(g.arg1ID, g.arg2ID) > 0) {
      //        int tmp = g.arg1ID;
      //        g.arg1ID = g.arg2ID;
      //        g.arg2ID = tmp;
      //        Set<String> list = g.arg1s;
      //        g.arg1s = g.arg2s;
      //        g.arg2s = list;
      //      }
      golds.add(g);
      files.add(g.docId);
    }
    reader.close();
    System.out.println(files.size() + " gold files");

    // pred
    List<Pred> preds = Lists.newArrayList();
    Path predFile = Paths.get("tmp/biocreative5/dataset/pred-20-title.csv");
    reader = new LineNumberReader(Files.newBufferedReader(predFile));
    //    reader.readLine();
    while ((line = reader.readLine()) != null) {
      Pred p = new Pred();
      String[] toks = line.split("\t");

      Set<PassageType> types = Splitter.on('|').splitToList(toks[3]).stream()
          .map(PassageType::valueOf)
          .collect(Collectors.toSet());

      if (files.contains(toks[0])){
//          && (types.contains(PassageType.TITLE_1) || types.contains(PassageType.TITLE_2)
//          || types.contains(PassageType.ABSTRACT) || types.contains(PassageType.FIG_CAPTION))) {
        p.docId = toks[0];
        if (!Documents20.contains(p.docId)) {
          continue;
        }

        p.arg1 = toks[1];
        p.arg2 = toks[2];
        p.status = Status.UNKNOWN;
        p.types = types;
        preds.add(p);
      }
    }
    reader.close();

    // eval
    List<Rst> rsts = Lists.newArrayList();
    for (Gold g : golds) {
      Rst rst = find(g, preds);
      checkNotNull(rst.types);
      rsts.add(rst);
    }
    for (Pred p : preds) {
      if (p.status == Status.UNKNOWN) {
        Rst rst = new Rst();
        rst.docId = p.docId;
        rst.arg1 = p.arg1;
        rst.arg2 = p.arg2;
        rst.status = Status.FP;
        rst.types = p.types;
        p.status = Status.FP;
        rsts.add(rst);
      }
    }

    PrecisionRecallStats stats = new PrecisionRecallStats();

    // eval
    Path rstFile = Paths.get("tmp/biocreative5/dataset/rst4-20-title.csv");
    BufferedWriter writer = Files.newBufferedWriter(rstFile);
    writer.write("DocId\tArg1\tArg2\tType\tStatus\n");
    for (Rst rst : rsts) {
      writer.write(rst.toString());
      writer.newLine();
      if (rst.status == Status.TP) {
        stats.incrementTP();
      } else if (rst.status == Status.FP) {
        stats.incrementFP();
      } else {
        stats.incrementFN();
      }
    }
    writer.close();

    System.out.println(stats);

    Multiset<String> fps = HashMultiset.create();
    for (Rst rst : rsts) {
      if (rst.status == Status.FP) {
        fps.add(rst.docId);
      }
    }
    for (String s : Multisets.copyHighestCountFirst(fps).elementSet()) {
      System.out.println(s + "\t" + fps.count(s));
    }
  }

  private static Rst find(Gold g, Collection<Pred> preds) {
    for (Pred p : preds) {
      if (isEqual(p, g)) {
        Rst rst = new Rst();
        rst.docId = p.docId;
        rst.arg1 = p.arg1;
        rst.arg2 = p.arg2;
        rst.status = Status.TP;
        rst.types = p.types;
        p.status = Status.TP;
        g.status = Status.TP;
        return rst;
      }
    }
    Rst rst = new Rst();
    rst.docId = g.docId;
    rst.status = Status.FN;
    rst.arg1 = Joiner.on('|').join(g.arg1s);
    rst.arg2 = Joiner.on('|').join(g.arg2s);
    rst.types = Sets.newHashSet();

    g.status = Status.FN;

    for (Pred p : preds) {
      if (p.docId.equals(g.docId)) {
        if (g.arg1s.contains(p.arg1)) {
          rst.arg1 = p.arg1;
        } else if (g.arg1s.contains(p.arg2)) {
          rst.arg1 = p.arg2;
        }
        if (g.arg2s.contains(p.arg2)) {
          rst.arg2 = p.arg2;
        } else if (g.arg2s.contains(p.arg1)) {
          rst.arg2 = p.arg1;
        }
        if (rst.arg1 != null && rst.arg2 != null) {
          break;
        }
      }
    }
    if (rst.arg1 != null && rst.arg2 != null) {
      rst.status = Status.FN_TWO;
      g.status = Status.FN_TWO;
    } else if (rst.arg1 != null || rst.arg2 != null) {
      rst.status = Status.FN_ONE;
      g.status = Status.FN_ONE;
    }

    return rst;
  }

  private static boolean isEqual(Pred p, Gold g) {
    return p.docId.equals(g.docId) &&
        ((contains(g.arg1s, p.arg1) && contains(g.arg2s, p.arg2))
            || (contains(g.arg1s, p.arg2) && contains(g.arg2s, p.arg1)));
  }

  // "p", "GTP", "GDP", "GFP", "gf", "egfp", "yfp", "myc", "gst", "ha", "DsRed"
  private static boolean contains(Set<String> set, String s) {
    for (String t : set) {
      if (t.equalsIgnoreCase(s)) {
        return true;
      }
      for (String tag : EpitopeString.EPITOPE_TAG) {
        if ((t + tag).equalsIgnoreCase(s)) {
          return true;
        }
        if ((t + "-" + tag).equalsIgnoreCase(s)) {
          return true;
        }
      }
    }

    return false;
  }

  private static class Rst {
    String docId;
    String arg1;
    String arg2;
    Status status;
    Set<PassageType> types;

    @Override
    public String toString() {
      StringJoiner sj = new StringJoiner("\t");
      sj.add(docId);
      if (arg1 != null) {
        sj.add(arg1);
      } else {
        sj.add("\t");
      }
      if (arg2 != null) {
        sj.add(arg2);
      } else {
        sj.add("\t");
      }
      sj.add(Joiner.on('|').join(types));
      sj.add(status.toString());
      return sj.toString();
    }
  }

  private static class Pred {
    String docId;
    String arg1;
    String arg2;
    Status status;
    Set<PassageType> types;
  }

  private static class Gold implements Comparable<Gold> {
    String docId;
    Set<String> arg1s;
    Set<String> arg2s;
    int arg1ID;
    int arg2ID;
    Status status;

    Gold() {
      arg1s = Sets.newHashSet();
      arg2s = Sets.newHashSet();
    }

    @Override
    public int compareTo(Gold gold) {
      int i = docId.compareTo(gold.docId);
      if (i == 0) {
        i = Integer.compare(arg1ID, gold.arg1ID);
      }
      if (i == 0) {
        i = Integer.compare(arg2ID, gold.arg2ID);
      }
      return i;
    }
  }

  enum Status {
    TP, FP, FN, FN_ONE, FN_TWO, UNKNOWN;
  }
}
