package com.pengyifan.experimental;

import com.google.common.base.Joiner;
import com.google.common.collect.Iterables;
import com.pengyifan.bioc.BioCAnnotation;
import com.pengyifan.bioc.BioCCollection;
import com.pengyifan.bioc.BioCDocument;
import com.pengyifan.bioc.BioCLocation;
import com.pengyifan.bioc.BioCPassage;
import com.pengyifan.bioc.util.BioCPassageIterator;
import com.pengyifan.bioc.util.BioCUtils;
import com.pengyifan.bioc.util.BioCValidate2;
import com.pengyifan.brat.BratDocument;
import com.pengyifan.brat.BratEntity;
import com.pengyifan.brat.io.BratIOUtils;
import com.pengyifan.commons.convert.BratEntity2Entity;
import com.pengyifan.commons.convert.Entity2BioCAnnotation;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Iterator;
import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;
import javax.xml.stream.XMLStreamException;
import org.apache.commons.io.FilenameUtils;

import static com.google.common.base.Preconditions.checkArgument;

public class Brat2BioC {

  public static void main(String[] args) throws IOException, XMLStreamException {
    new Brat2BioC().doMain(args);
  }

  public void doMain(String[] args) throws IOException, XMLStreamException {
    Path idir = Paths.get("tmp/biocreative5/dataset/20-document/bioc-shift/");
    Path idir2 = Paths.get("tmp/biocreative5/dataset/20-document/gold");
    Path odir = Paths.get("tmp/biocreative5/dataset/20-document/bioc-gold2");

    final Function<BratEntity, BioCAnnotation> function =
        new BratEntity2Entity.BasicConverter().andThen(
            new Entity2BioCAnnotation.BasicConverter()
        );

    for (Path p : Files.newDirectoryStream(idir, "*.xml")) {
      System.out.println(p);

      String basename = FilenameUtils.getBaseName(p.getFileName().toString());
      Path i = idir2.resolve(basename + ".ann");
      BratDocument bratDocument = BratIOUtils.read(Files.newBufferedReader(i), basename);

      BioCCollection c = BioCUtils.readCollection(p);
      BioCDocument bioCDocument = Iterables.getOnlyElement(c.getDocuments());

      List<BioCAnnotation> annotationList = bratDocument.getEntities().stream()
          .map(function::apply)
          .collect(Collectors.toList());

      BioCPassageIterator itr = new BioCPassageIterator(c);
      while (itr.hasNext()) {
        BioCPassage passage = itr.next();
        if (!passage.getText().isPresent()) {
          continue;
        }
        int start = passage.getOffset();
        int end = start + passage.getText().get().length();

        Iterator<BioCAnnotation> annitr = annotationList.iterator();
        while (annitr.hasNext()) {
          BioCAnnotation ann = annitr.next();
          BioCLocation location = ann.getTotalLocation();
          if (start <= location.getOffset() && location.getOffset() + location.getLength() <= end) {
//            String expected = ann.getText().get();
//            String actual = an
            passage.addAnnotation(ann);
            annitr.remove();
          }
        }

      }

      try {
        checkArgument(annotationList.isEmpty(), "%s\n%s", p, Joiner.on('\n').join(annotationList));
      } catch (Exception e) {
        System.err.println(e.getMessage());
      }
      BioCValidate2.check(c);

      //
      //      for(BratEntity ent: bratDocument.getEntities()) {
      //        BioCAnnotation ann = function.apply(ent);
      //        bioCDocument.addAnnotation(ann);
      //      }

      Path o = odir.resolve(FilenameUtils.getBaseName(p.getFileName().toString()) + ".xml");
      BioCUtils.writeCollectioin(o, c);
    }
  }
  //
  //  public BioCAnnotation convert(BratEntity bratEntity) {
  //    BioCAnnotation biocAnnotatioin = new BioCAnnotation();
  //    biocAnnotatioin.setID(bratEntity.getId());
  //    biocAnnotatioin.setText(bratEntity.getText());
  //    for (Range<Integer> range : bratEntity.getSpans().asRanges()) {
  //      biocAnnotatioin.addLocation(new BioCLocation(range.lowerEndpoint(), range
  //          .upperEndpoint() - range.lowerEndpoint()));
  //    }
  //    Map<String, String> infons = Maps.newHashMap();
  //    infons.put("type", bratEntity.getType());
  //    biocAnnotatioin.setInfons(infons);
  //    return biocAnnotatioin;
  //  }
}
