package com.pengyifan.experimental;

import com.google.common.base.Joiner;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import com.pengyifan.brat.BratDocument;
import com.pengyifan.brat.BratEntity;
import com.pengyifan.brat.BratRelation;
import com.pengyifan.brat.io.BratIOUtils;
import org.apache.commons.io.FilenameUtils;

import java.io.BufferedWriter;
import java.io.IOException;
import java.nio.file.DirectoryStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Map;
import java.util.Set;

public class CountUniqPPI {
  public static void main(String[] args) throws IOException {

    Map<C, Set<String>> map = Maps.newTreeMap();

    Path dir = Paths.get("tmp/aimed/dataset/ppi-basic/");
    for(Path f: Files.newDirectoryStream(dir)) {
      collect(f, map);
    }
    dir = Paths.get("tmp/aimed/dataset/ppi-complex/");
    for(Path f: Files.newDirectoryStream(dir)) {
      collect(f, map);
    }
    dir = Paths.get("tmp/aimed/dataset/ppi-extended/");
    for(Path f: Files.newDirectoryStream(dir)) {
      collect(f, map);
    }
    dir = Paths.get("tmp/aimed/dataset/ppi-tech/");
    for(Path f: Files.newDirectoryStream(dir)) {
      collect(f, map);
    }

    Path rstFile = Paths.get("tmp/aimed/dataset/pred-subtype.csv");
    BufferedWriter w = Files.newBufferedWriter(rstFile);
    for(C c: map.keySet()) {
      w.write(String.format("%s\t%s\t%s\t%s\n", c.doc, c.arg1, c.arg2,
          Joiner.on(',').join(map.get(c))));
    }
    w.close();
  }

  public static void collect(Path f, Map<C, Set<String>> map) throws IOException {
    String basename = FilenameUtils.getBaseName(f.toString());
    BratDocument bratDocument = BratIOUtils.read(Files.newBufferedReader(f), basename);

    String docId = basename;//basename.substring(0, basename.indexOf('_'));
    for (BratRelation r: bratDocument.getRelations()) {
      String arg1 = ((BratEntity)bratDocument.getAnnotation(r.getArgId("Arg1"))).getText();
      String arg2 = ((BratEntity)bratDocument.getAnnotation(r.getArgId("Arg2"))).getText();
      C c = new C();
      c.doc = docId;
      if (arg1.compareTo(arg2) > 0) {
        c.arg1 = arg2;
        c.arg2 = arg1;
      } else {
        c.arg1 = arg1;
        c.arg2 = arg2;
      }
      if (map.containsKey(c)) {
        map.get(c).add(r.getType());
      } else {
        Set<String> set = Sets.newTreeSet();
        set.add(r.getType());
        map.put(c, set);
      }
    }
  }

  private static class C implements Comparable<C> {
    String doc;
    String arg1;
    String arg2;


    @Override
    public int compareTo(C c) {
      int i = doc.compareTo(c.doc);
      if (i == 0) {
        i = arg1.compareTo(c.arg1);
      }
      if (i == 0) {
        i = arg2.compareTo(c.arg2);
      }
      return i;
    }
  }
}
