
package com.pengyifan.experimental;

import com.google.common.base.Joiner;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import edu.stanford.nlp.stats.PrecisionRecallStats;

import java.io.BufferedWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import java.util.Set;

public class EvalAimed {
  public static void main(String[] args) throws IOException {

    Set<G> golds = Sets.newTreeSet();

    Path goldFile = Paths.get("tmp/aimed/dataset/gold-sen.csv");
    for(String line: Files.readAllLines(goldFile)) {
      try {
        String[] toks = line.split("\t");
        G c = new G();
        c.doc = docId(toks[0]);
        c.arg1 = toks[1];
        c.arg2 = toks[2];
        golds.add(c);
      } catch (Exception e) {
        System.err.println(line);
      }
    }

    Set<P> preds = Sets.newTreeSet();

    Path predFile = Paths.get("tmp/aimed/dataset/pred-subtype.csv");
    for(String line: Files.readAllLines(predFile)) {
      String [] toks = line.split("\t");
      P c = new P();
      c.doc = docId(toks[0]);
      c.arg1 = toks[1];
      c.arg2 = toks[2];
      c.types.addAll(Lists.newArrayList(toks[3].split(",")));
      preds.add(c);
    }

    PrecisionRecallStats stats = new PrecisionRecallStats();
    List<G> goldlist = Lists.newArrayList(golds);
    boolean [] bs = new boolean[goldlist.size()];

    List<R> result = Lists.newArrayList();

    for(P p: preds) {
      int index = -1;
      for(int i=0; i<goldlist.size(); i++) {
        G g = goldlist.get(i);
        if (p.doc.equals(g.doc)
            && (p.arg1.equalsIgnoreCase(g.arg1) && p.arg2.equalsIgnoreCase(g.arg2)
            || p.arg1.equalsIgnoreCase(g.arg1) && p.arg2.equalsIgnoreCase(g.arg2))) {
          stats.incrementTP();
          index = i;
          bs[i] = true;
          result.add(new R(p, "tp"));
          break;
        }
      }
      if (index == -1) {
        result.add(new R(p, "fp"));
        stats.incrementFP();
      }
    }
    for(int i=0; i<bs.length; i++) {
      if (!bs[i]) {
        result.add(new R(goldlist.get(i), "fn"));
        stats.incrementFN();
      }
    }

    Path rstFile = Paths.get("tmp/aimed/dataset/rst-sen2.csv");
    BufferedWriter w = Files.newBufferedWriter(rstFile);
    for (R r: result) {
      w.write(Joiner.on('\t').join(r.doc, r.arg1, r.arg2, Joiner.on(',').join(r.types), r.status));
      w.newLine();
    }
    w.close();

    System.out.println(stats);
    System.out.println(stats.getPrecision());
    System.out.println(stats.getRecall());
    System.out.println(stats.getF1Description(4));
  }

  private static String docId(String docId) {
    //int index = docId.lastIndexOf('.');
    //return docId.substring(0, index);
    return docId;
  }

  private static class G implements Comparable<G> {
    String doc;
    String arg1;
    String arg2;

    @Override
    public int compareTo(G c) {
      int i = doc.compareTo(c.doc);
      if (i == 0) {
        i = arg1.compareTo(c.arg1);
      }
      if (i == 0) {
        i = arg2.compareTo(c.arg2);
      }
      return i;
    }
  }

  private static class P implements Comparable<P> {
    String doc;
    String arg1;
    String arg2;
    Set<String> types;

    P () {
      types = Sets.newTreeSet();
    }

    @Override
    public int compareTo(P c) {
      int i = doc.compareTo(c.doc);
      if (i == 0) {
        i = arg1.compareTo(c.arg1);
      }
      if (i == 0) {
        i = arg2.compareTo(c.arg2);
      }
      return i;
    }
  }

  private static class R {
    String doc;
    String arg1;
    String arg2;
    Set<String> types;
    String status;

    R () {
      types = Sets.newTreeSet();
    }

    R(G g, String status) {
      this();
      doc = g.doc;
      arg1 = g.arg1;
      arg2 = g.arg2;
      this.status = status;
    }

    R(P g, String status) {
      doc = g.doc;
      arg1 = g.arg1;
      arg2 = g.arg2;
      types = g.types;
      this.status = status;
    }
  }
}

