package com.pengyifan.experimental;

import com.google.common.base.Joiner;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import com.pengyifan.brat.BratDocument;
import com.pengyifan.brat.BratEntity;
import com.pengyifan.brat.BratRelation;
import com.pengyifan.brat.io.BratIOUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.StringReader;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class GetAIMedGold {
  public static void main(String[] args) throws IOException {
    Set<C> set = Sets.newTreeSet();

    Path dir = Paths.get(System.getProperty("user.home")
        + "/Dropbox/Researches/data/aimed/whole-bySentence/preprocessed/brat-withNegative/");
    for(Path f: Files.newDirectoryStream(dir, "*.a1")) {
      collect(f, set);
    }

    Path rstFile = Paths.get("tmp/aimed/dataset/gold-sen.csv");
    BufferedWriter w = Files.newBufferedWriter(rstFile);
    for(C c: set) {
      w.write(String.format("%s\t%s\t%s\n", c.doc, c.arg1, c.arg2));
    }
    w.close();
  }

  public static void collect(Path f, Set<C> set) throws IOException {
    System.out.println(f);
    String basename = FilenameUtils.getBaseName(f.toString());

    String a1 = FileUtils.readFileToString(f.toFile());
    String a2 = FileUtils.readFileToString(f.getParent().resolve(basename + ".a2").toFile());

    Pattern p = Pattern.compile("#.*?\n");
    Matcher m = p.matcher(a2);
    a2 = m.replaceAll("");

    BratDocument bratDocument = BratIOUtils.read(new StringReader(a1 + "\n" + a2), basename);

    String docId = basename;//basename.substring(0, basename.indexOf('_'));
    for (BratRelation r: bratDocument.getRelations()) {
      if (r.getType().equals("Interaction-True")) {
        String arg1 = ((BratEntity) bratDocument.getAnnotation(r.getArgId("Arg1"))).getText();
        String arg2 = ((BratEntity) bratDocument.getAnnotation(r.getArgId("Arg2"))).getText();
        C c = new C();
        c.doc = docId;
        if (arg1.compareTo(arg2) > 0) {
          c.arg1 = arg2;
          c.arg2 = arg1;
        } else {
          c.arg1 = arg1;
          c.arg2 = arg2;
        }
        set.add(c);
      }
    }
  }

  private static class C implements Comparable<C> {
    String doc;
    String arg1;
    String arg2;


    @Override
    public int compareTo(C c) {
      int i = doc.compareTo(c.doc);
      if (i == 0) {
        i = arg1.compareTo(c.arg1);
      }
      if (i == 0) {
        i = arg2.compareTo(c.arg2);
      }
      return i;
    }
  }
}
