package com.pengyifan.experimental;

import com.google.common.base.Joiner;
import com.google.common.collect.Lists;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import java.util.Objects;
import com.pengyifan.commons.math.PrecisionRecallStats;

import static com.google.common.base.Preconditions.checkArgument;

public class PPIEval {
  public static void main(String[] args) throws IOException {
    Path goldFile = Paths.get("tmp/biocreative5/dataset/simple.csv");
    Path predFile = Paths.get("tmp/biocreative5/dataset/pred.csv");

    List<PPI> golds = Lists.newArrayList();
    for (String line : Files.readAllLines(goldFile)) {
      String[] toks = line.toLowerCase().split("\t");
      PPI ppi = new PPI(toks[0], toks[1], toks[2]);
      golds.add(ppi);
    }

    List<PPI> preds = Lists.newArrayList();
    for (String line : Files.readAllLines(predFile)) {
      String[] toks = line.toLowerCase().split("\t");
      PPI ppi = new PPI(toks[0], toks[1], toks[2]);
      preds.add(ppi);
    }

    PrecisionRecallStats<String> stats1 = new PrecisionRecallStats<>();
    for (PPI gold : golds) {
      boolean b = false;
      for (PPI pred : preds) {
        if (equal(gold, pred)) {
          stats1.incrementTP(gold.toString());
          b = true;
          break;
        }
      }
      if (!b) {
        stats1.incrementFN(gold.toString());
      }
    }
    PrecisionRecallStats stats2 = new PrecisionRecallStats();
    for (PPI pred : preds) {
      boolean b = false;
      for (PPI gold : golds) {
        if (equal(gold, pred)) {
          stats2.incrementTP(gold.toString());
          b = true;
          break;
        }
      }
      if (!b) {
        stats2.incrementFP(pred.toString());
      }
    }
//    checkArgument(stats1.getTP()==stats2.getTP());

    PrecisionRecallStats stats = new PrecisionRecallStats(
        stats1.getTPs(),
        stats2.getFPs(),
        stats1.getFNs());
    System.out.println(stats);
    System.out.println("Prep:  " + stats.getPrecision());
    System.out.println("Recall:" + stats.getRecall());
    System.out.println("Fvalue:" + stats.getFMeasure());

    Files.write(Paths.get("tmp/biocreative5/dataset/tps.txt"), stats.getTPs());
    Files.write(Paths.get("tmp/biocreative5/dataset/fps.txt"), stats.getFPs());
    Files.write(Paths.get("tmp/biocreative5/dataset/fns.txt"), stats.getFNs());
  }

  public static boolean equal(PPI ppi1, PPI ppi2) {
    return (ppi1.docId.equals(ppi2.docId)
        && ppi1.arg1.equals(ppi2.arg1)
        && ppi1.arg2.equals(ppi2.arg2))
        || (ppi1.docId.equals(ppi2.docId)
        && ppi1.arg1.equals(ppi2.arg2)
        && ppi1.arg2.equals(ppi2.arg1));
  }

  public static class PPI {
    public final String docId;
    public final String arg1;
    public final String arg2;

    public PPI(String docId, String arg1, String arg2) {
      this.docId = docId;
      this.arg1 = arg1;
      this.arg2 = arg2;
    }

    @Override
    public boolean equals(Object obj) {
      if (this == obj) {
        return true;
      }
      if (!(obj instanceof PPI)) {
        return false;
      }
      PPI rhs = (PPI) obj;
      return Objects.equals(docId, rhs.docId)
          && Objects.equals(arg1, rhs.arg1)
          && Objects.equals(arg2, rhs.arg2);
    }

    @Override
    public int hashCode() {
      return Objects.hash(docId, arg1, arg2);
    }

    @Override
    public String toString() {
      return Joiner.on('\t').join(docId, arg1, arg2);
    }
  }
}
