package com.pengyifan.experimental;

import com.google.common.collect.Lists;
import com.pengyifan.bioc.BioCAnnotation;
import com.pengyifan.bioc.BioCCollection;
import com.pengyifan.bioc.BioCPassage;
import com.pengyifan.bioc.util.BioCPassageIterator;
import com.pengyifan.bioc.util.BioCUtils;
import com.pengyifan.bioc.util.BioCValidate2;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import javax.xml.stream.XMLStreamException;
import org.apache.commons.io.FilenameUtils;

public class ValidateBioC {

  public static void main(String[] args) throws IOException, XMLStreamException {
    new ValidateBioC().doMain(args);
  }

  public void doMain(String[] args) throws IOException, XMLStreamException {
    Path idir = Paths.get("tmp/biocreative5/dataset/20-document/bioc-gold/");
    Path odir = Paths.get("tmp/biocreative5/dataset/20-document/bioc-gold2");

    for (Path p : Files.newDirectoryStream(idir, "*.xml")) {
      System.out.println(p);

      BioCCollection c = BioCUtils.readCollection(p);
      BioCPassageIterator itr = new BioCPassageIterator(c);
      int index = 0;
      while (itr.hasNext()) {
        BioCPassage passage = itr.next();
        List<BioCAnnotation> annotations = Lists.newArrayList(passage.getAnnotations());
        passage.clearAnnotations();
        annotations.sort((a1, a2) -> Integer.compare(
            a1.getTotalLocation().getOffset(), a2.getTotalLocation().getOffset()));
        for (BioCAnnotation annotation : annotations) {
          annotation.setID("T" + index++);
        }
        passage.clearAnnotations();
        annotations.forEach(passage::addAnnotation);
      }
      BioCValidate2.check(c);
      Path o = odir.resolve(FilenameUtils.getBaseName(p.getFileName().toString()) + ".xml");
      BioCUtils.writeCollectioin(o, c);
    }
  }
}
