package com.pengyifan.kernel;

import com.google.common.collect.Lists;
import com.pengyifan.brat.BratEntity;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraph;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraphEdge;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraphVertex;
import com.pengyifan.commons.collections.dependencygraph.ShortestPathDirection;
import com.pengyifan.commons.math.SparseRealVector;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import static com.google.common.base.Preconditions.checkNotNull;

public class PairInstance {

  public String id;
  public double goldValue;
  public double predValue;

  public PositiveNegativeStatus positiveNegativeStatus;
  public BratEntity entity1;
  public BratEntity entity2;
  public DependencyGraph dependencyGraph;
  public DependencyGraph sequenceGraph;
  public DependencyGraph ssdg;
  public DependencyGraph shortestPathGraph;
  public List<List<DependencyGraphEdge>> paths;
  public Integer startIndex;
  public Integer endIndex;
  public String text;
  public List<Integer> range1IndexList;
  public List<Integer> range2IndexList;
  // public List<String> pathStrings;

  public SparseRealVector vector;
  public List<LinkedList<Object>> listOfPathObjects;
  public String libsvmLine;

  // feature
  public Map<String, Double> bows;
  public Map<String, Double> vwalks;
  public Map<String, Double> ewalks;
  public Map<String, Double> apgFeatures;
  public Map<String, Double> seqFeatures;


  public PairInstance() {
  }

  public void buildListOfPathObjects() {

    checkNotNull(ssdg, "ssdg is not set");
    checkNotNull(paths, "Paths is not set");
    checkNotNull(startIndex, "start index is not set");
    checkNotNull(endIndex, "end index is not set");

    listOfPathObjects = Lists.newArrayList();

    if (paths.isEmpty()) {
      return;
    }

    for (List<DependencyGraphEdge> path : paths) {

      LinkedList<Object> pathObjects = Lists.newLinkedList();
      pathObjects.add(ssdg.getVertex(startIndex));
      for (DependencyGraphEdge e : path) {
        DependencyGraphVertex target = ssdg.getEdgeTarget(e);
        if (e.getShortestPathDirection() == ShortestPathDirection.UP) {
          target = ssdg.getEdgeSource(e);
        }
        String arrow = e.getShortestPathDirection().toPathString();
        pathObjects.add(arrow);
        pathObjects.add(e);
        pathObjects.add(arrow);
        pathObjects.add(target);
      }
      listOfPathObjects.add(pathObjects);
    }
  }
}
