package com.pengyifan.kernel;

import com.google.common.base.Joiner;
import com.pengyifan.commons.lang.IndentStringFormatter;
import java.io.BufferedWriter;
import java.io.Closeable;
import java.io.IOException;
import java.io.Writer;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;

public class PairInstancePrinter implements Closeable {

  public static void writeInstances(Path file, List<PairInstance> instances)
      throws IOException {
    PairInstancePrinter printer = new PairInstancePrinter(Files.newBufferedWriter(file));
    for (PairInstance instance : instances) {
      printer.write(instance);
    }
    printer.close();
  }

  private static final IndentStringFormatter formatter2 = IndentStringFormatter.newFormat()
      .withHangIndent(true).withIndent(6).withWidth(100);
  private BufferedWriter writer;

  public PairInstancePrinter(Writer writer) {
    this.writer = new BufferedWriter(writer);

  }

  public void write(PairInstance instance) throws IOException {
    String line = Joiner.on('\t').skipNulls().join(instance.id,
        instance.positiveNegativeStatus);
    line += "\n      ";
    writer.append(line);
    writer.newLine();
    // get text
    writer.append(formatter2.format("TEXT: " + instance.text));
    writer.newLine();
    writer.newLine();
  }

  @Override
  public void close() throws IOException {
    writer.close();
  }
}
