package com.pengyifan.kernel.allpathgraph.singleprocess;

import com.pengyifan.commons.collections.dependencygraph.Dependency;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraph;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraphEdge;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraphVertex;
import com.pengyifan.commons.collections.dependencygraph.DependencyName;
import com.pengyifan.commons.collections.dependencygraph.NamedEntity;
import com.pengyifan.commons.collections.dependencygraph.PartOfSpeech;
import com.pengyifan.commons.collections.dependencygraph.Position;
import edu.stanford.nlp.ling.IndexedWord;
import edu.stanford.nlp.semgraph.SemanticGraph;
import edu.stanford.nlp.semgraph.SemanticGraphEdge;
import java.util.logging.Logger;

import static com.google.common.base.Preconditions.checkNotNull;
import static com.google.common.base.Preconditions.checkState;

public class DependencyGraphBuilder2 {

  private static final Logger LOGGER = Logger.getAnonymousLogger();

  final SemanticGraph semanticGraph;

  public DependencyGraphBuilder2(SemanticGraph semanticGraph) {
    this.semanticGraph = semanticGraph;
  }

  public DependencyGraph build() {

    DependencyGraph graph = new DependencyGraph();

    // nodes
    for (IndexedWord node : semanticGraph.vertexListSorted()) {
      if (!graph.containsVertex(node.index())) {
        graph.addVertex(createVertex(node));
      }
    }

    // edges
    int edgeIndex = graph.getMaxIndex() + 1;
    for (SemanticGraphEdge edge : semanticGraph.edgeListSorted()) {
      int sourceIndex = edge.getSource().index();
      int targetIndex = edge.getTarget().index();

      // remove self loop
      if (sourceIndex == targetIndex) {
        LOGGER.info(String.format("Skip self-loop[%s] in graph", edge));
        continue;
      }

      checkState(graph.containsVertex(sourceIndex),
          "cannot find gov: %s\n%s\n%s", sourceIndex, edge, semanticGraph);
      checkState(graph.containsVertex(targetIndex),
          "cannot find dep: %s\n%s\n%s", targetIndex, edge, semanticGraph);
      checkNotNull(edge.getRelation().getShortName(), "no dep name: %s", edge);

      DependencyGraphVertex source = graph.getVertex(sourceIndex);
      DependencyGraphVertex target = graph.getVertex(targetIndex);
      if (!graph.containsEdge(source, target)) {
        graph.addEdge(source, target, createEdge(edgeIndex, edge));
        edgeIndex++;
      }
    }
    return graph;
  }

  private DependencyGraphEdge createEdge(int index,
      SemanticGraphEdge edge) {
    return DependencyGraphEdge.of(
        index,
        new Dependency(DependencyName.valueOf(edge.getRelation().getShortName().toUpperCase()),
        edge.getRelation().getSpecific()));
  }

  private DependencyGraphVertex
      createVertex(IndexedWord indexedWord) {
    return new DependencyGraphVertex(
        indexedWord.index(),
        indexedWord.word(),
        PartOfSpeech.getEnum(indexedWord.tag()),
        indexedWord.lemma(),
        indexedWord.ner() == null ? NamedEntity.UNKNOWN
            : NamedEntity.valueOf(indexedWord.ner()),
        Position.UNKNOWN,
        indexedWord.beginPosition(),
        indexedWord.endPosition());
  }
}
