package com.pengyifan.kernel.allpathgraph.singleprocess;

import com.google.common.collect.Lists;
import com.pengyifan.commons.collections.Pairs;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraph;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraphEdge;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraphVertex;
import java.util.LinkedList;
import java.util.List;
import org.javatuples.Pair;

import static com.google.common.base.Preconditions.checkState;

public class DuplicatedLinkChecker {

  private final DependencyGraph graph;

  public DuplicatedLinkChecker(DependencyGraph graph) {
    this.graph = graph;
  }

  public void check() {
    List<DependencyGraphVertex> vertexList = Lists.newArrayList(graph
        .vertexSet());
    for (int i = 0; i < vertexList.size(); i++) {
      DependencyGraphVertex v1 = vertexList.get(i);
      for (int j = i; j < vertexList.size(); j++) {
        DependencyGraphVertex v2 = vertexList.get(j);
        checkDuplicates(v1, v2);
        checkDuplicates(v2, v1);
      }
    }
  }

  private void removeDuplicates() {
    List<DependencyGraphVertex> vertexList = Lists.newArrayList(graph
        .vertexSet());
    for (int i = 0; i < vertexList.size(); i++) {
      DependencyGraphVertex v1 = vertexList.get(i);
      for (int j = i; j < vertexList.size(); j++) {
        DependencyGraphVertex v2 = vertexList.get(j);
        // removeEdges(v1, v2);
        // removeEdges(v2, v1);
      }
    }
  }

  private void checkDuplicates(DependencyGraphVertex v1,
      DependencyGraphVertex v2) {
    for(Pair<DependencyGraphEdge, DependencyGraphEdge> pair: Pairs.getPairs(graph.getAllEdges(v1, v2))) {
      DependencyGraphEdge e1 = pair.getValue0();
      DependencyGraphEdge e2 = pair.getValue1();
      boolean b = e1.getDependency().getName() == e2.getDependency().getName()
          && e1.getDependency().getSpecific().equals(e2.getDependency().getSpecific());
      if (b) {
        throw new IllegalArgumentException(String.format("Mutiple edges: %s", pair));
      }
    }
  }

  private void removeEdges(DependencyGraphVertex v1, DependencyGraphVertex v2) {
    LinkedList<DependencyGraphEdge> edgeList = Lists.newLinkedList(graph
        .getAllEdges(
            v1,
            v2));

    while (edgeList.size() > 1) {
      DependencyGraphEdge e1 = edgeList.pop();
      DependencyGraphEdge e2 = edgeList.pop();
      if (e1.getDependency().getName().equals(e2.getDependency().getName())
          && e1.getDependency().getSpecific().equals(e2.getDependency().getSpecific())) {
        edgeList.push(e1);
      } else if ("ref".equals(e1.getDependency().getName())
          && !"ref".equals(e2.getDependency().getName())) {
        edgeList.push(e1);
      } else if (!"ref".equals(e1.getDependency().getName())
          && "ref".equals(e2.getDependency().getName())) {
        edgeList.push(e2);
      } else if (!"conj".equals(e1.getDependency().getName())
          && "conj".equals(e2.getDependency().getName())) {
        edgeList.push(e1);
      } else if ("conj".equals(e1.getDependency().getName())
          && !"conj".equals(e2.getDependency().getName())) {
        edgeList.push(e2);
      } else if (tryToPush(e1, e2, "nn", "poss", edgeList)) {
        ;
      } else if (tryToPush(e1, e2, "cop", "auxpass", edgeList)) {
        ;
      } else if (tryToPush(e1, e2, "mwe", "pobj", edgeList)) {
        ;
      } else if (tryToPush(e1, e2, "dobj", "iobj", edgeList)) {
        ;
      } else if (tryToPush(e1, e2, "parataxis", "dobj", edgeList)) {
        ;
      } else if (tryToPush(e1, e2, "nsubj", "prep", edgeList)) {
        ;
      } else if ("part-whole".equals(e1.getDependency().getSpecific().orElse(null))
          && "denotation".equals(e2.getDependency().getSpecific().orElse(null))) {
        edgeList.push(e1);
      } else if ("denotation".equals(e1.getDependency().getSpecific().orElse(null))
          && "part-whole".equals(e2.getDependency().getSpecific().orElse(null))) {
        edgeList.push(e2);
      } else {
        edgeList.push(e1);
        // checkState(false, "more than one edge left:\n%s\n%s", e1, e2);
      }
    }

    checkState(edgeList.size() <= 1, "more than one edge left:\n%s", edgeList);
    if (!edgeList.isEmpty()) {
      DependencyGraphEdge e = edgeList.get(0);
      for (DependencyGraphEdge edge : graph.getAllEdges(v1, v2)) {
        if (edge != e) {
          graph.removeEdge(edge);
        }
      }
    }
  }

  private boolean tryToPush(DependencyGraphEdge e1, DependencyGraphEdge e2,
      String name1, String name2, LinkedList<DependencyGraphEdge> edgeList) {
    if (name1.equals(e1.getDependency().getName())
        && name2.equals(e2.getDependency().getName())) {
      edgeList.push(e1);
      return true;
    } else if (name2.equals(e1.getDependency().getName())
        && name1.equals(e2.getDependency().getName())) {
      edgeList.push(e2);
      return true;
    } else {
      return false;
    }
  }
}
