package com.pengyifan.nlp.process;

import com.pengyifan.nlp.trees.PtbWriter;
import com.pengyifan.util.PropertiesLoader;
import edu.stanford.nlp.io.FileSequentialCollection;
import edu.stanford.nlp.ling.CoreAnnotations.SentencesAnnotation;
import edu.stanford.nlp.pipeline.Annotation;
import edu.stanford.nlp.pipeline.StanfordCoreNLP;
import edu.stanford.nlp.trees.TreeCoreAnnotations.TreeAnnotation;
import edu.stanford.nlp.util.CoreMap;
import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Collection;
import java.util.Properties;
import java.util.logging.LogManager;
import java.util.logging.Logger;

public class Parse {

  public static void main(String[] args)
      throws Exception {
    LogManager.getLogManager().readConfiguration(
        PropertiesLoader.loadLoggingPropertiesStream());

    String corpusName = "biocreative2";

    Path prefix = Paths.get(
        "/home/ypeng/Dropbox/Researches/data",
        corpusName,
        "bc2");

    Parse s = new Parse(
        prefix.resolve("brat"),
        prefix.resolve("stanford/ptb1")
    );
    s.process();
  }

  private static Logger logger = Logger.getAnonymousLogger();
  private static final int NUM_THREADS = 8;

  private final StanfordCoreNLP pipeline;
  private final Path inputDir;

  public Parse(Path inputDir, Path outputDir) {
    this.inputDir = inputDir;
    Properties props = new Properties();
    props.put("annotators", "tokenize, ssplit, parse");
    props.put("ssplit.newlineIsSentenceBreak", "always");
    props.put("parse.flags", "-retainTmpSubcategories");
    props.put("threads", Integer.toString(NUM_THREADS));
    props.put("inputDirectory", inputDir.toString());
    props.put("extension", ".txt");
    props.put("outputDirectory", outputDir.toString());
    props.put("outputExtension", ".ptb");
    props.put("outputFormat", "text");
    props.put("replaceExtension", "true");

    logger.info(String.format("Parse props: %s", props));
    pipeline = new MyStanfordCoreNLP(props);
  }

  public void process()
      throws IOException {
    Properties props = pipeline.getProperties();
    Collection<File> files = new FileSequentialCollection(inputDir.toFile(),
        props.getProperty("extension"), false);
    pipeline.processFiles(null, files, NUM_THREADS);
  }

  private static class MyStanfordCoreNLP extends StanfordCoreNLP {

    private MyStanfordCoreNLP(Properties props) {
      super(props);
    }

    /**
     * only print tree.
     */
    @Override
    public void prettyPrint(Annotation annotation, OutputStream os) {
      PrintWriter writer = new PrintWriter(os);
      prettyPrint(annotation, writer);
    }

    /**
     * only print tree.
     */
    @Override
    public void prettyPrint(Annotation annotation, PrintWriter writer) {
      for (CoreMap sentence : annotation.get(SentencesAnnotation.class)) {
        writer.println(PtbWriter.print(sentence.get(TreeAnnotation.class)));
      }
      writer.flush();
    }
  }
}
