package com.pengyifan.nlp.process;

import static com.google.common.base.Preconditions.checkState;

import com.pengyifan.nlp.trees.TreeUtils;

import edu.stanford.nlp.ling.CoreLabel;
import edu.stanford.nlp.trees.Tree;
import edu.stanford.nlp.trees.Treebank;

public class PtbAlignment {

  private final String text;
  private final Treebank treebank;

  public PtbAlignment(String text, Treebank treebank) {
    this.text = text;
    this.treebank = treebank;
  }

  public final void alignment()
      throws Exception {
    // alignment tree
    int textIndex = 0;
    for (Tree tree : treebank) {
      for (Tree tn : tree.postOrderNodeList()) {

        CoreLabel label = (CoreLabel) tn.label();
        String value = TreeUtils.adaptValue(tn.value());

        if (tn.isLeaf()) {
          int from = text.indexOf(value, textIndex);
          if (from == -1) {
            from = text.indexOf(tn.value(), textIndex);
          }
          int to = from + value.length();

          checkState(
              from != -1,
              "cannot find word %s from %d\n[tree=%s]\n[from=%d]%s\n%s",
              value,
              textIndex,
              tree,
              textIndex,
              text.substring(textIndex),
              text);

          textIndex = to;

          label.setBeginPosition(from);
          label.setEndPosition(to);
          label.setWord(label.value());
        } else if (tn.isPreTerminal()) {
          label.setTag(label.value());
        } else if (tn.isPhrasal()) {
          label.setCategory(label.value());
        }
      }
    }

  }
}
