package com.pengyifan.nlp.ssdg;

import java.util.List;

import com.google.common.collect.Iterables;
import com.pengyifan.brat.BratDocument;
import com.pengyifan.brat.BratEntity;
import com.pengyifan.brat.BratRelation;
import com.pengyifan.commons.collections.dependencygraph.Dependency;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraph;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraphEdge;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraphVertex;
import com.pengyifan.commons.collections.dependencygraph.DependencyName;
import com.pengyifan.kernel.allpathgraph.singleprocess.DependencyGraphVertexFinder;

class AbbrevAdder {

  private final DependencyGraph graph;
  private final BratDocument abbrDoc;

  public AbbrevAdder(DependencyGraph graph, BratDocument abbrDoc) {
    this.graph = graph;
    this.abbrDoc = abbrDoc;
  }

  /**
   * If referee is protein, add edge from referer to referee;
   */
  public int add() {
    int startIndex = graph.getMaxIndex() + 1;
    int edgeIndex = startIndex;
    for (BratRelation r : abbrDoc.getRelations()) {
      // short-form
      BratEntity shortForm = abbrDoc.getEntity(r.getArgId("short-form"));
      List<DependencyGraphVertex> shortFormVertexList =
          new DependencyGraphVertexFinder(graph, shortForm.totalSpan()).find();
      if (shortFormVertexList.size() != 1) {
        continue;
      }

      // long-form
      BratEntity longForm = abbrDoc.getEntity(r.getArgId("long-form"));
      List<DependencyGraphVertex> longFormVertexList =
          new DependencyGraphVertexFinder(graph, longForm.totalSpan()).find();
      if (longFormVertexList.isEmpty()) {
        continue;
      }

      DependencyGraphVertex source = shortFormVertexList.get(0);
      DependencyGraphVertex target = Iterables.getLast(longFormVertexList);
      if (source == target) {
        continue;
      }

      if (!graph.containsEdge(source, target,
          Dependency.ofPredicate(DependencyName.IS_A, "abbr"))) {
        graph.addEdge(source, target, DependencyGraphEdge.of(edgeIndex,
            new Dependency(DependencyName.IS_A, "abbr")));
        edgeIndex++;
      }
    }
    return edgeIndex - startIndex;
  }
}
