package com.pengyifan.nlp.ssdg;

import java.util.function.Predicate;

import com.pengyifan.commons.collections.dependencygraph.Dependency;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraph;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraphEdge;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraphUtils;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraphVertex;
import com.pengyifan.commons.collections.dependencygraph.DependencyName;

class ConjLinkPropagation {

  private final DependencyGraph graph;

  public ConjLinkPropagation(DependencyGraph graph) {
    this.graph = graph;
  }

  public int propagate() {
    int numOfEdges = 0;
    numOfEdges += propagateDash();
    // numOfEdges += propagateConj();
    numOfEdges += propagateBesides();
    // propagateParataxis();
    return numOfEdges;
  }

  @SuppressWarnings("unused")
  private int propagateConj() {
    int numOfEdges = 0;
    numOfEdges += propagateConj("and");
    // numOfEdges += propagate(DependencyName.CONJ, "or");
    // numOfEdges += propagate(DependencyName.CONJ, "and/or");
    return numOfEdges;
  }

  private int propagateBesides() {
    int startIndex = graph.getMaxIndex() + 1;
    int edgeIndex = startIndex;

    final Predicate<DependencyGraphEdge> predicate =
        e -> e.getDependency().equals(DependencyName.PREP, "besides");

    for (DependencyGraphEdge e : graph.edgeSet(predicate)) {
      DependencyGraphVertex target = graph.getEdgeTarget(e);
      DependencyGraphVertex source = graph.getEdgeSource(e);
      if (graph.hasOutgoingEdgesOf(source, Dependency.ofPredicate(DependencyName.ARG1))
          && !graph.containsEdge(source, target,
              Dependency.ofPredicate(DependencyName.ARG1, "besides"))) {
        edgeIndex++;
        graph.addEdge(source, target,
            DependencyGraphEdge.of(edgeIndex, new Dependency(DependencyName.ARG1, "besides")));
      }
    }
    return edgeIndex - startIndex;
  }

  // private int propagateParataxis() {
  // return propagate(DependencyName.PARATAXIS, null);
  // }

  private int propagateDash() {
    int numOfEdges = 0;

    final Predicate<DependencyGraphEdge> predicate =
        e -> e.getDependency().equals(DependencyName.CONJ, "dash");
    for (DependencyGraphEdge e : graph.edgeSet(predicate)) {
      DependencyGraphVertex target = graph.getEdgeTarget(e);
      DependencyGraphVertex source = graph.getEdgeSource(e);
      numOfEdges += propagateFromSourceSourceToTarget(source, target);
    }
    return numOfEdges;
  }

  private int propagateConj(String dependencySpecific) {
    int numOfEdges = 0;

    final Predicate<DependencyGraphEdge> predicate =
        e -> e.getDependency().equals(DependencyName.CONJ, dependencySpecific);
    for (DependencyGraphEdge e : graph.edgeSet(predicate)) {
      DependencyGraphVertex target = graph.getEdgeTarget(e);
      DependencyGraphVertex source = graph.getEdgeSource(e);
      // dobj
      if (graph.hasOutgoingEdgesOf(target, Dependency.ofPredicate(DependencyName.DOBJ))
          && !graph.hasOutgoingEdgesOf(source, Dependency.ofPredicate(DependencyName.DOBJ))) {
        numOfEdges += propagateFromSourceToTargetTarget(source, target);
      }
      // numOfEdges +=
      // DependencyGraphUtils.propagateFromTargetSourceToSource(
      // graph,
      // e,
      // source,
      // target);
      // numOfEdges +=
      // DependencyGraphUtils.propagateFromSourceToTargetTarget(
      // graph,
      // source,
      // target);
      // numOfEdges +=
      // DependencyGraphUtils.propagateFromTargetToSourceTarget(
      // graph,
      // source,
      // target);
    }
    return numOfEdges;
  }

  /**
   * <pre>
   *             (add)
   *   +-------------------------+
   *   |                         |
   *   |                         v
   * source --> target --> target target
   * </pre>
   */
  private int propagateFromSourceToTargetTarget(
      DependencyGraphVertex source,
      DependencyGraphVertex target) {
    int startIndex = graph.getMaxIndex() + 1;
    int edgeIndex = startIndex;
    for (DependencyGraphEdge outgoingEdgeOfTarget : graph.outgoingEdgesOf(target)) {
      DependencyGraphVertex targetTarget = graph.getEdgeTarget(outgoingEdgeOfTarget);
      DependencyGraphEdge edge = new DependencyGraphEdge(edgeIndex, outgoingEdgeOfTarget);
      if (targetTarget != source
          && !DependencyGraphUtils.containsEdge(graph, source, targetTarget, edge)) {
        graph.addEdge(source, targetTarget, edge);
        edgeIndex++;
      }
    }
    return edgeIndex - startIndex;
  }

  /**
   * <pre>
   *                 (add)
   *       +------------------------+
   *       |                        |
   *       |                        v
   * source source --> source --> target
   * </pre>
   */
  private int propagateFromSourceSourceToTarget(
      DependencyGraphVertex source,
      DependencyGraphVertex target) {
    int startIndex = graph.getMaxIndex() + 1;
    int edgeIndex = startIndex;
    for (DependencyGraphEdge incomingEdgeOfSource : graph.incomingEdgesOf(source)) {
      DependencyGraphVertex sourceSource = graph.getEdgeSource(incomingEdgeOfSource);
      DependencyGraphEdge edge = new DependencyGraphEdge(edgeIndex, incomingEdgeOfSource);
      if (sourceSource != target
          && !DependencyGraphUtils.containsEdge(graph, sourceSource, target, edge)) {
        graph.addEdge(sourceSource, target, edge);
        edgeIndex++;
      }
    }
    return edgeIndex - startIndex;
  }
}
