package com.pengyifan.nlp.ssdg;

import java.util.Set;

import com.google.common.collect.Sets;
import com.pengyifan.commons.collections.dependencygraph.Dependency;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraph;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraphPattern;
import com.pengyifan.commons.collections.dependencygraph.DependencyName;

class DescriptiveLinkAdder extends EdgeAdder {

  DescriptiveLinkAdder(DependencyGraph graph) {
    super(graph);
  }

  private static final Set<String> withKeywords = Sets.newHashSet(
      "those"
      );

  private static final Set<String> ofKeywords = Sets.newHashSet(
      "amount",
      "both",
      "specificity"
  );

  private static boolean isKeyWord(String keyword, Set<String> set) {
    return set.contains(keyword);
  }

  int add() {
    int numOfEdges = 0;
    numOfEdges += addPrepWith();
    numOfEdges += addPrepOf();
    return numOfEdges;
  }

  /**
   * descriptive > with > nn
   */
  private static final DependencyGraphPattern prepWithPattern = DependencyGraphPattern
      .of()
      // descriptive
      .addVertex(0, v -> isKeyWord(v.getLemma(), withKeywords))
      // nn
      .addVertex(1, v -> v.getTag().isNoun())
      .addEdge(0, 1, Dependency.ofPredicate(DependencyName.PREP, "with"));

  private int addPrepWith() {
    return add(prepWithPattern, 0, 1, DependencyName.DESCRIPTIVE, "with");
  }

  /**
   * descriptive > of > nn
   */
  private static final DependencyGraphPattern prepOfPattern = DependencyGraphPattern
      .of()
          // descriptive
      .addVertex(0, v -> isKeyWord(v.getLemma(), ofKeywords))
          // nn
      .addVertex(1, v -> v.getTag().isNoun())
      .addEdge(0, 1, Dependency.ofPredicate(DependencyName.PREP, "with"));

  private int addPrepOf() {
    return add(prepOfPattern, 0, 1, DependencyName.DESCRIPTIVE, "of");
  }
}
