package com.pengyifan.nlp.ssdg;

import com.pengyifan.commons.collections.dependencygraph.Dependency;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraph;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraphEdge;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraphMatcher;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraphPattern;
import com.pengyifan.commons.collections.dependencygraph.DependencyGraphVertex;
import com.pengyifan.commons.collections.dependencygraph.DependencyName;

public abstract class EdgeAdder {

  protected final DependencyGraph graph;

  public EdgeAdder(DependencyGraph graph) {
    this.graph = graph;
  }
  
  abstract int add();

  protected int add(DependencyGraphPattern pattern,
      int source,
      int target,
      DependencyName name,
      String specific) {
    int startIndex = graph.getMaxIndex() + 1;
    int edgeIndex = startIndex;
    DependencyGraphMatcher matcher = pattern.matcher(graph);
    while (matcher.find()) {
      DependencyGraphVertex s = matcher.get(source);
      DependencyGraphVertex t = matcher.get(target);
      edgeIndex = addIfNotExist(s, t, name, specific, edgeIndex);
    }
    return edgeIndex - startIndex;
  }

  protected int addIfNotExist(DependencyGraphVertex source,
      DependencyGraphVertex target,
      DependencyName name,
      String specific,
      int index) {
    if (!graph.containsEdge(source, target, Dependency.ofPredicate(name, specific))) {
      graph.addEdge(source, target, DependencyGraphEdge.of(index, new Dependency(name, specific)));
      index++;
    }
    return index;
  }

}