package com.pengyifan.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Properties;
import org.apache.commons.configuration.ConfigurationConverter;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;

public final class PropertiesLoader {

  private static final String LOG_PROPERTIES = "logging.properties";

  private PropertiesLoader() {
  }

  // TODO Remove unused code found by UCDetector
  //   /**
  //    * Load properties from <code>src/main/resources</code>. Properties can
  //    * reference other properties
  //    */
  //   public static Properties loadProperties(String filename)
  //       throws IOException, ConfigurationException {
  //     return loadProperties(getInputStream(filename));
  //   }

  /**
   * Load properties from absolute path. Properties can reference other
   * properties
   */
  public static Properties loadProperties(File path)
      throws IOException, ConfigurationException {
    return loadProperties(getInputStream(path));
  }

  private static Properties loadProperties(InputStream in)
      throws IOException, ConfigurationException {
    PropertiesConfiguration config = new PropertiesConfiguration();
    config.load(in, StandardCharsets.UTF_8.name());
    return ConfigurationConverter.getProperties(config);
  }

  private static InputStream getInputStream(String filename) {
    return Thread.currentThread().getContextClassLoader()
        .getResourceAsStream(filename);
  }

  private static InputStream getInputStream(File path)
      throws FileNotFoundException {
    return new FileInputStream(path);
  }

  public static InputStream loadLoggingPropertiesStream()
      throws IOException {
    return getInputStream(LOG_PROPERTIES);
  }
}
