package com.pengyifan.commons.collections.dependencygraph;

import static org.junit.Assert.assertEquals;

import org.junit.Before;

public class DependencyGraphTest {

  private static final DependencyGraphVertex GSVA = getVertex(1, 'A');
  private static final DependencyGraphVertex GSVB = getVertex(2, 'B');
  private static final DependencyGraphVertex GSVC = getVertex(3, 'C');
  private static final DependencyGraphVertex GSVD = getVertex(4, 'D');
  private static final DependencyGraphVertex GSVE = getVertex(5, 'E');
  private static final DependencyGraphEdge GSEAB = getEdge(10, DependencyName.ACOMP);
  private static final DependencyGraphEdge GSEBC = getEdge(11, DependencyName.ADVCL);
  private static final DependencyGraphEdge GSECD = getEdge(12, DependencyName.AGENT);
  private static final DependencyGraphEdge GSEEC = getEdge(13, DependencyName.AMOD);

  private DependencyGraph graph;

  /**
   * <pre>
   * A --> B --> C --> D
   *             ^
   *             |
   *       E ----+
   * </pre>
   */
  @Before
  public void setUp() {
    graph = new DependencyGraph();
    graph.addVertex(GSVA);
    graph.addVertex(GSVB);
    graph.addVertex(GSVC);
    graph.addVertex(GSVD);
    graph.addVertex(GSVE);
    graph.addEdge(GSVA, GSVB, GSEAB);
    graph.addEdge(GSVB, GSVC, GSEBC);
    graph.addEdge(GSVC, GSVD, GSECD);
    graph.addEdge(GSVE, GSVC, GSEEC);
  }

//  @Test
//  @Ignore
//  public void testAddEdgeFromSourceSourceToTarget() {
//    int i = DependencyGraphUtils.propagateFromSourceSourceToTarget(
//        graph,
//        GSVB,
//        GSVC);
//    assertEquals(1, i);
//    testEdgeName("AB", graph.getEdge(GSVA, GSVC));
//  }
//
//  @Test
//  @Ignore
//  public void testAddEdgeFromTargetToSourceTarget() {
//    int i = DependencyGraphUtils.propagateFromTargetToSourceTarget(
//        graph,
//        GSVB,
//        GSVD);
//    assertEquals(1, i);
//    testEdgeName("BC", graph.getEdge(GSVD, GSVC));
//  }
//
//  @Test
//  @Ignore
//  public void testAddEdgeFromTargetSourceToSource() {
//    int i = DependencyGraphUtils.propagateFromTargetSourceToSource(
//        graph,
//        GSEBC,
//        GSVB,
//        GSVC);
//    assertEquals(1, i);
//    testEdgeName("EC", graph.getEdge(GSVE, GSVC));
//  }
//
//  @Test
//  @Ignore
//  public void testAddEdgeFromSourceToTargetTarget() {
//    int i = DependencyGraphUtils.propagateFromSourceToTargetTarget(
//        graph,
//        GSVB,
//        GSVC);
//    assertEquals(1, i);
//    testEdgeName("CD", graph.getEdge(GSVB, GSVD));
//  }

  private void testEdgeName(String expected, DependencyGraphEdge edge) {
    assertEquals(expected, edge.getDependency().getName());
  }

  private static DependencyGraphVertex getVertex(int index, char c) {
    DependencyGraphVertex v = new DependencyGraphVertex(index);
    v.setWord(Character.toString(c));
    return v;
  }

  private static DependencyGraphEdge getEdge(int index, DependencyName name) {
    return DependencyGraphEdge.of(index, new Dependency(name, null));
  }
}
